@@ -26,12 +26,15 @@
  *
  *  The central file responsible for managing the graphical user interface.
  */
+
 #include "../i18n.h"
 #include "../global.h"
 #include "../SimStateHandler.h"
 #include "crrc_gui_main.h"
 #include "crrc_gui_viewwind.h"
+#include "crrc_dialog.h"
 #include "crrc_msgbox.h"
+#include "puaImageFrame.h"
 #include "crrc_planesel.h"
 #include "crrc_video.h"
 #include "crrc_launch.h"
@@ -45,6 +48,7 @@
 #include "crrc_ctrlgen.h"
 #include "crrc_audio.h"
 #include "crrc_location.h"
+#include "crrc_f3a.h"
 #include "crrc_f3f.h"
 #include "crrc_loadrobot.h"
 #include "crrc_setrecordname.h"
@@ -52,6 +56,7 @@
 #include "../mod_misc/filesystools.h"
 #include "../mod_misc/lib_conversions.h"
 #include "../mod_video/crrc_graphics.h"
+#include "../mod_video/fonts.h"
 
 
 #ifdef linux
@@ -72,7 +77,9 @@ static void view_fullsc_cb(puObject*);
 static void view_train_cb(puObject*);
 static void view_testmode_cb(puObject*);
 static void view_hudcompass_cb(puObject*);
+static void view_windvectors_cb(puObject*);
 static void view_verbosity_cb(puObject*);
+static void view_modelwindow_cb(puObject*);
 static void view_zoomin_cb(puObject*);
 static void view_zoomout_cb(puObject*);
 static void view_unzoom_cb(puObject*);
@@ -87,7 +94,7 @@ static void opt_video_cb(puObject*);
 static void opt_audio_cb(puObject*);
 static void opt_ctrl_general_cb(puObject*);
 
-
+static void game_f3a_cb(puObject*);
 static void game_f3f_cb(puObject*);
 
 static void help_web_cb(puObject*);
@@ -98,8 +105,6 @@ static void robot_load_cb(puObject*);
 static void robot_rm_all_cb(puObject*);
                                        
 
-#define VERBOSITY_FONT_FILE "textures/Helvetica_iso8859-15.txf"
-
 /** \brief Create the GUI object.
  *
  *  Creates the GUI and sets its "visible" state.
@@ -125,17 +130,21 @@ CGUIMain::CGUIMain(bool vis) : visible(vis)
   const char *view_submenu[]   = { _("Inspect Wind"),
                                    _("Reset Zoom"),_("Zoom -"), 
                                    _("Zoom +"), _("Toggle Verbosity"),
+                                   _("Toggle Wind vectors"),
                                    _("Toggle HUD Compass"),
                                    _("Toggle Test Mode"),
                                    _("Toggle Training Mode"),
+                                   _("Toggle Model Window Mode"),
                                    _("Toggle Fullscreen"),
                                    NULL};
   puCallback view_submenu_cb[] = {  view_wind_cb,
                                     view_unzoom_cb, view_zoomout_cb,
                                     view_zoomin_cb, view_verbosity_cb,
+                                    view_windvectors_cb,
                                     view_hudcompass_cb,
                                     view_testmode_cb,
                                     view_train_cb,
+                                    view_modelwindow_cb,                     
                                     view_fullsc_cb,
                                     NULL};
   // Simulation menu
@@ -155,8 +164,8 @@ CGUIMain::CGUIMain(bool vis) : visible(vis)
                                   opt_location_cb, opt_plane_cb, NULL};
 
   // Game menu
-  const char *game_submenu[]   = {"F3F", NULL};
-  static puCallback game_submenu_cb[] = {game_f3f_cb, NULL};
+  const char *game_submenu[]   = {"F3A", "F3F", NULL};
+  static puCallback game_submenu_cb[] = {game_f3a_cb, game_f3f_cb, NULL};
 
   // Help menu
   const char *help_submenu[]   = {_("About"), _("Keys"), _("Help"), NULL};
@@ -181,15 +190,9 @@ CGUIMain::CGUIMain(bool vis) : visible(vis)
   verboseOutput = new puText(30, 30);
   verboseOutput->setColour(PUCOL_LABEL, 1, 0.1, 0.1);
 
-  VerbosityFont = new fntTexFont ();
-  std::string fname = FileSysTools::getDataPath(VERBOSITY_FONT_FILE);
-  if ((fname == "") || (VerbosityFont->load( fname.c_str() ) == FNT_FALSE))
+  if (Video::textureFont)
   {
-    std::cout << "CGUIMain: Unable to find font " << VERBOSITY_FONT_FILE << ", falling back to bitmap font!" << std::endl;
-  }
-  else
-  {
-    verboseOutput->setLabelFont(VerbosityFont);
+    verboseOutput->setLabelFont(Video::textureFont);
   }
   verboseOutput->reveal();
 
@@ -200,7 +203,7 @@ CGUIMain::CGUIMain(bool vis) : visible(vis)
     compass_x[i]->setColour(PUCOL_LABEL, 1., 0.1, 0.1, 0.7);
     compass_x[i]->setLabelPlace(PUPLACE_BOTTOM_CENTERED);
     if (Video::window_xsize > 800)
-      compass_x[i]->setLabelFont(PUFONT_HELVETICA_18); 
+      //compass_x[i]->setLabelFont(PUFONT_HELVETICA_18); 
     compass_x[i]->setLabel(""); 
     compass_x[i]->hide();
   }
@@ -210,7 +213,7 @@ CGUIMain::CGUIMain(bool vis) : visible(vis)
     compass_y[i]->setColour(PUCOL_LABEL, 1., 0.1, 0.1, 0.7);
     compass_y[i]->setLabelPlace(PUPLACE_CENTERED_RIGHT); 
     if (Video::window_xsize > 800)
-      compass_y[i]->setLabelFont(PUFONT_HELVETICA_18); 
+      //compass_y[i]->setLabelFont(PUFONT_HELVETICA_18); 
     compass_y[i]->setLabel(""); 
     compass_y[i]->hide();
   }
@@ -229,7 +232,6 @@ CGUIMain::CGUIMain(bool vis) : visible(vis)
     else
       SDL_ShowCursor(SDL_DISABLE);
   }
-
 }
 
 
@@ -515,9 +517,16 @@ void CGUIMain::doHUDCompass(const float field_of_view)
   if (Global::HUDCompass && !isVisible())
   {
     // compass visible..
-    
-    puFont stdFont = Video::window_xsize > 800 ? PUFONT_HELVETICA_18 : PUFONT_9_BY_15; 
-    puFont bigFont = Video::window_xsize > 800 ? PUFONT_TIMES_ROMAN_24 : PUFONT_HELVETICA_18; 
+    puFont stdFont, bigFont;
+    if (Video::textureFont)
+    {
+      stdFont = puFont ( Video::textureFont, Video::window_xsize > 800 ? 20 : 14 );
+      bigFont = puFont ( Video::textureFont, Video::window_xsize > 800 ? 24 : 18 );
+    }
+    else{    
+      stdFont = Video::window_xsize > 800 ? PUFONT_HELVETICA_18 : PUFONT_9_BY_15; 
+      bigFont = Video::window_xsize > 800 ? PUFONT_TIMES_ROMAN_24 : PUFONT_HELVETICA_18;
+    }
     CRRCMath::Vector3 look_dir = Video::looking_pos - player_pos;
     float azimuth = atan2(look_dir.r[0], -look_dir.r[2])*180.0/M_PI;
     float elevation = atan2(look_dir.r[1], sqrt(pow(look_dir.r[0],2) + pow(look_dir.r[2],2)))*180.0/M_PI;
@@ -724,6 +733,11 @@ static void opt_ctrl_general_cb(puObject *obj)
   new CGUICtrlGeneralDialog();
 }
 
+static void game_f3a_cb(puObject *obj)
+{
+  new CGUIF3ADialog();
+}
+
 static void game_f3f_cb(puObject *obj)
 {
   new CGUIF3FDialog();
@@ -748,33 +762,160 @@ static void help_web_cb(puObject *obj)
 
 static void help_keys_cb(puObject *obj)
 {
-  std::string help_txt = _("Key mapping:\n");
-  help_txt += "\n";
-  (help_txt += "ESC    ") += _("show/hide menu\n");
-  (help_txt += "q      ") += _("quit\n");
-  (help_txt += "r      ") += _("restarts after crash\n");
-  (help_txt += "p      ") += _("pause/resume simulation\n");
-  (help_txt += "c      ") += _("reload model configuration\n");
-  (help_txt += "d      ") += _("toggle control input debugging mode\n");
-  (help_txt += "t      ") += _("toggle training mode which displays the location of the thermals\n");
-  (help_txt += "v      ") += _("toggle verbosity level (0..3) to display control inputs/FOV/FPS\n");
-  (help_txt += "h      ") += _("toggle HUD compass visualisation mode\n");
-  (help_txt += "g      ") += _("toggle landing gear (if function is not mapped to a controller)\n");
-  (help_txt += "b      ") += _("toggle spoiler/airbrake (if function is not mapped to a controller)\n");
-  (help_txt += "pg-up  ") += _("increase throttle (if you aren't using JOYSTICK or better)\n");
-  (help_txt += "pg-dwn ") += _("decrease throttle (if you aren't using JOYSTICK or better)\n");
-  (help_txt += _("left/right arrow ")) += _("rudder\n");
-  (help_txt += _("up/down arrow    ")) += _("elevator\n");
-  (help_txt += "+      ") += _("zoom in (assuming zoom.control is KEYBOARD)\n");
-  (help_txt += "-      ") += _("zoom out (assuming zoom.control is KEYBOARD)\n");
-   help_txt += "\n\n";
- 
-  new CGUIMsgBox(help_txt.c_str());
+  const int keyw = 125;
+  const int txtw = 500;
+  int y_in_win = 1 * DLG_DEF_BUTTON_HEIGHT + 2 * DLG_DEF_SPACE;
+  int dy = DLG_CHECK_H + 0.4 * DLG_DEF_SPACE;
+  
+  CRRCDialog *KeysDialog = new CRRCDialog(0, 0, CRRC_DIALOG_OK);
+
+  puText *txta01 = new puText(DLG_DEF_SPACE, y_in_win);
+  txta01->setLabel("-");
+  puText *txtb01 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb01->setLabel(_("zoom out (assuming zoom.control is KEYBOARD)"));
+
+  puText *txta02 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta02->setLabel("+");
+  puText *txtb02 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb02->setLabel(_("zoom in (assuming zoom.control is KEYBOARD)"));
+
+  puText *txta03 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta03->setLabel(_("up/down arrow"));
+  puText *txtb03 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb03->setLabel(_("elevator"));
+
+  puText *txta04 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta04->setLabel(_("left/right arrow"));
+  puText *txtb04 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb04->setLabel(_("rudder"));
+
+  puText *txta05 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta05->setLabel(_("pg-dwn"));
+  puText *txtb05 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb05->setLabel(_("decrease throttle (if you aren't using JOYSTICK or better)"));
+
+  puText *txta06 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta06->setLabel(_("pg-up"));
+  puText *txtb06 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb06->setLabel(_("increase throttle (if you aren't using JOYSTICK or better)"));
+
+  puText *txta07 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta07->setLabel("b");
+  puText *txtb07 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb07->setLabel(_("toggle spoiler/airbrake (if function is not mapped to a controller)"));
+
+  puText *txta08 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta08->setLabel("g");
+  puText *txtb08 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb08->setLabel(_("toggle landing gear (if function is not mapped to a controller)"));
+
+  puText *txta09 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta09->setLabel("w");
+  puText *txtb09 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb09->setLabel(_("toggle 3D wind vector visualisation mode (0..2)"));
+
+  puText *txta10 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta10->setLabel("h");
+  puText *txtb10 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb10->setLabel(_("toggle HUD compass visualisation mode"));
+
+  puText *txta11 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta11->setLabel("v");
+  puText *txtb11 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb11->setLabel(_("toggle verbosity level (0..3) to display control inputs/FOV/FPS"));
+
+  puText *txta12 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta12->setLabel("t");
+  puText *txtb12 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb12->setLabel(_("toggle training mode which displays the location of the thermals"));
+
+  puText *txta13 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta12->setLabel("m");
+  puText *txtb13 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb12->setLabel(_("toggle model's close-view window visualisation mode (0..2)"));
+
+  puText *txta14 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta13->setLabel("d");
+  puText *txtb14 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb13->setLabel(_("toggle control input debugging mode"));
+
+  puText *txta15 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta14->setLabel("c");
+  puText *txtb15 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb14->setLabel(_("reload model configuration"));
+
+  puText *txta16 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta15->setLabel("p");
+  puText *txtb16 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb15->setLabel(_("pause/resume simulation"));
+
+  puText *txta17 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta16->setLabel("r");
+  puText *txtb17 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb16->setLabel(_("restart after crash"));
+
+  puText *txta18 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta17->setLabel("q");
+  puText *txtb18 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb17->setLabel(_("quit"));
+
+  puText* __attribute__((unused)) txta19 = new puText(DLG_DEF_SPACE, y_in_win += dy);
+  txta18->setLabel("ESC");
+  puText* __attribute__((unused)) txtb19 = new puText(keyw + DLG_DEF_SPACE, y_in_win);
+  txtb18->setLabel(_("show/hide menu"));
+
+  puText *txta00 = new puText(DLG_DEF_SPACE, y_in_win += dy + DLG_DEF_SPACE);
+  txta00->setLabel(_("Key mapping:\n"));
+
+  y_in_win += dy;
+  
+
+  KeysDialog->close(); 
+  KeysDialog->setSize(1 * keyw + 1 * txtw + 3 * DLG_DEF_SPACE,
+                      1 * y_in_win        + 1 * DLG_DEF_SPACE);
+  KeysDialog->centerOnScreen();
+  KeysDialog->reveal();  
 }
 
 static void help_about_cb(puObject *obj)
 {
-  new CGUIMsgBox(PACKAGE_STRING);
+  const int txtw = 200;
+  const int imgw = 128;
+  const int imgh = 128;
+  int y_in_win = 1 * DLG_DEF_BUTTON_HEIGHT + 2 * DLG_DEF_SPACE;
+  static ssgTexture *image = NULL;
+  
+  CRRCDialog *AboutDialog = new CRRCDialog(0, 0, CRRC_DIALOG_OK);
+
+  // package identification: string needs to be persistent after construction
+  static std::string package_txt;
+  package_txt = PACKAGE_NAME;
+  package_txt += "\n\n";
+  package_txt += _("version info:\n");
+  package_txt += PACKAGE_VERSION;
+  puText *txt01 = new puText(2 * DLG_DEF_SPACE + imgw, y_in_win + imgh);
+  txt01->setLabel(package_txt.c_str());
+  txt01->setLabelPlace(PUPLACE_BELOW_RIGHT);
+  
+  // load "About" image file
+  if (!image)
+  {
+    std::string imageAboutFilename = FileSysTools::getDataPath("textures/Crrcsim.jpg");
+    image = new ssgTexture(imageAboutFilename.c_str());
+  }
+
+  // frame widget to store image
+  new puaImageFrame (
+    1*DLG_DEF_SPACE,        y_in_win,
+    1*DLG_DEF_SPACE + imgw, y_in_win + imgh,
+    image
+  );
+
+  AboutDialog->close(); 
+  AboutDialog->setSize(1 * imgw + 1 * txtw + 2 * DLG_DEF_SPACE,
+                       1 * imgh + 1 * DLG_DEF_BUTTON_HEIGHT + 3 * DLG_DEF_SPACE);
+  AboutDialog->centerOnScreen();
+  AboutDialog->reveal();  
 }
 
 static void view_fullsc_cb(puObject *obj)
@@ -834,6 +975,22 @@ static void view_hudcompass_cb(puObject *obj)
     Global::HUDCompass = 1;
 }
 
+static void view_windvectors_cb(puObject *obj)
+{
+  if (Global::windVectors == 2)
+    Global::windVectors = 0;
+  else
+    Global::windVectors++;
+}
+
+static void view_modelwindow_cb(puObject *obj)
+{
+  if (Global::modelView == 2)
+    Global::modelView = 0;
+  else
+    Global::modelView++;
+}
+
 static void view_zoomin_cb(puObject *obj)
 {
   zoom_in();
@@ -860,8 +1017,8 @@ static void quitDialogCallback(int choice)
 
 static void view_testmode_cb(puObject *obj)
 {
-  Global::testmode.test_mode = (Global::testmode.test_mode ^ 0x0001) & 0x0001;
-  if (Global::testmode.test_mode)
+  Global::testmode = !Global::testmode;
+  if (Global::testmode)
     activate_test_mode();
   else
     leave_test_mode();
