@@ -27,16 +27,17 @@
 #include "../i18n.h"
 #include "../global.h"
 #include "../aircraft.h"
-#include "crrc_gui_main.h"
-#include "crrc_planesel.h"
 #include "../crrc_main.h"
 #include "../crrc_loadair.h"
+#include "../SimStateHandler.h"
 #include "../mod_misc/filesystools.h"
 #include "../mod_misc/lib_conversions.h"
 #include "../mod_fdm/fdm.h"
 #include "../mod_fdm/formats/airtoxml.h"
 #include "../mod_fdm/xmlmodelfile.h"
 #include "util.h"
+#include "crrc_gui_main.h"
+#include "crrc_planesel.h"
 
 #include <iostream>
 #include <sys/types.h>
@@ -108,18 +109,28 @@ CGUIPlaneSelectDialog::CGUIPlaneSelectDialog()
   description->setText("This is a short description of the selected model.");
 
   // Use launch default check box
-  check_usedefault = new puButton(x1,             y2-4*DLG_DEF_BUTTON_HEIGHT-DLG_DEF_SPACE,             // x, y lower left
-                                  x1+DLG_CHECK_W, y2-4*DLG_DEF_BUTTON_HEIGHT-DLG_DEF_SPACE+DLG_CHECK_H  // x, y upper right
-                                  );
-  check_usedefault->setButtonType(PUBUTTON_VCHECK);
-  check_usedefault->setLabelPlace(PUPLACE_CENTERED_RIGHT);
-  check_usedefault->setLabel(_("Load launch default"));
-  check_usedefault->setValue((int)cfgfile->getInt("airplane.use_default_launch", 1));
-  check_usedefault->greyOut();
+  check_useLaunchDefault = new puButton(x1,             y2-4*DLG_DEF_BUTTON_HEIGHT-DLG_DEF_SPACE,             // x, y lower left
+                                        x1+DLG_CHECK_W, y2-4*DLG_DEF_BUTTON_HEIGHT-DLG_DEF_SPACE+DLG_CHECK_H  // x, y upper right
+                                       );
+  check_useLaunchDefault->setButtonType(PUBUTTON_VCHECK);
+  check_useLaunchDefault->setLabelPlace(PUPLACE_CENTERED_RIGHT);
+  check_useLaunchDefault->setLabel(_("Load launch default"));
+  check_useLaunchDefault->setValue((int)cfgfile->getInt("airplane.use_default_launch", 1));
+  check_useLaunchDefault->greyOut();
+
+  // Use mixer default check box
+  check_useMixerDefault = new puButton(x2,             y2-4*DLG_DEF_BUTTON_HEIGHT-DLG_DEF_SPACE,             // x, y lower left
+                                       x2+DLG_CHECK_W, y2-4*DLG_DEF_BUTTON_HEIGHT-DLG_DEF_SPACE+DLG_CHECK_H  // x, y upper right
+                                      );
+  check_useMixerDefault->setButtonType(PUBUTTON_VCHECK);
+  check_useMixerDefault->setLabelPlace(PUPLACE_CENTERED_RIGHT);
+  check_useMixerDefault->setLabel(_("Load mixer default"));
+  check_useMixerDefault->setValue((int)cfgfile->getInt("airplane.use_default_mixer", 1));
+  check_useMixerDefault->greyOut();
 
   // Graphics selection box
-  gbox = new puaComboBox(x1,               y2-1*DLG_DEF_BUTTON_HEIGHT-2*msg_height, // x, y lower left
-                         x2-DLG_DEF_SPACE, y2-0*DLG_DEF_BUTTON_HEIGHT-2*msg_height, // x, y upper right
+  gbox = new puaComboBox(x1,                   y2-1*DLG_DEF_BUTTON_HEIGHT-2*msg_height, // x, y lower left
+                         x2-0.5*DLG_DEF_SPACE, y2-0*DLG_DEF_BUTTON_HEIGHT-2*msg_height, // x, y upper right
                          NULL,
                          false);
   gbox->setChildColourScheme(PUCLASS_POPUPMENU, dlgCol1[0], dlgCol1[1], dlgCol1[2]);
@@ -131,8 +142,8 @@ CGUIPlaneSelectDialog::CGUIPlaneSelectDialog()
   gbox->setCallback(CGUIPlaneSelGraphicsCallback);
   
   // Configuration selection box
-  cbox = new puaComboBox(x2, y2-1*DLG_DEF_BUTTON_HEIGHT-2*msg_height, // x, y lower left
-                         x3, y2-0*DLG_DEF_BUTTON_HEIGHT-2*msg_height, // x, y upper right
+  cbox = new puaComboBox(x2+0.5*DLG_DEF_SPACE, y2-1*DLG_DEF_BUTTON_HEIGHT-2*msg_height, // x, y lower left
+                         x3,                   y2-0*DLG_DEF_BUTTON_HEIGHT-2*msg_height, // x, y upper right
                          NULL,
                          false);
   cbox->setChildColourScheme(PUCLASS_POPUPMENU, dlgCol1[0], dlgCol1[1], dlgCol1[2]);
@@ -213,10 +224,7 @@ CGUIPlaneSelectDialog::CGUIPlaneSelectDialog()
   close();
   setSize(dw, dh);
   setCallback(CGUIPlaneSelCallback);
-  
-  // center the dialog on screen
   centerOnScreen();
-
   reveal();
 }
 
@@ -581,26 +589,41 @@ void CGUIPlaneSelectDialog::updateFileInfo()
     location_label_string += plane_paths[entry];
     location_label->setLabel(location_label_string.c_str());
     
-    // Update the description box and launch default checkbox
+    // Update the description box and launch/mixer default checkboxes
     description_string = "";
     try
     {
       SimpleXMLTransfer *model = new SimpleXMLTransfer(plane_paths[entry]);
       description_string = model->getChild("description.en")->getContentString();
-      // The test for a "launch" child shouldn't throw an exception,
+
+      // The test for a "launch" or "mixer" child shouldn't throw an exception,
       // so we let it create the child if the test fails and then
-      // test for its children to see if it was a real "launch" tag
+      // test for its children to see if it was a real tag
+      
       SimpleXMLTransfer *launch = model->getChild("launch", true);
       if (launch->getChildCount() > 0)
       {
-        check_usedefault->setValue((int)cfgfile->getInt("airplane.use_default_launch", 1));
-        check_usedefault->activate();
+        check_useLaunchDefault->setValue((int)cfgfile->getInt("airplane.use_default_launch", 1));
+        check_useLaunchDefault->activate();
       }
       else
       {
-        check_usedefault->setValue(1);  // always use "default" if there are no options
-        check_usedefault->greyOut();
+        check_useLaunchDefault->setValue(0);  // unchecked if there are no options
+        check_useLaunchDefault->greyOut();
       }
+
+      SimpleXMLTransfer *mixer = model->getChild("mixer", true);
+      if (mixer->getChildCount() > 0)
+      {
+        check_useMixerDefault->setValue((int)cfgfile->getInt("airplane.use_default_mixer", 1));
+        check_useMixerDefault->activate();
+      }
+      else
+      {
+        check_useMixerDefault->setValue(0);  // unchecked if there are no options
+        check_useMixerDefault->greyOut();
+      }
+      
       delete model;
     }
     catch (XMLException e)
@@ -833,6 +856,8 @@ void CGUIPlaneSelCallback(puObject *obj)
 
       cfgfile->setAttributeOverwrite("airplane.use_default_launch",
                                       (dlg->getLoadLaunchDefault() == 0) ? "0" : "1");
+      cfgfile->setAttributeOverwrite("airplane.use_default_mixer",
+                                      (dlg->getLoadMixerDefault() == 0) ? "0" : "1");
       
       try
       {
@@ -866,13 +891,33 @@ void CGUIPlaneSelCallback(puObject *obj)
                                            def_launch->getString("rel_front", doubleToString(MODELSTART_REL_FRONT)));
             cfgfile->setAttributeOverwrite("launch.rel_right", 
                                            def_launch->getString("rel_right", doubleToString(MODELSTART_REL_RIGHT)));
+          }          
+        }
+        
+        // check if the user wants to load the default mixer settings
+        // for this airplane
+        if (dlg->getLoadMixerDefault() == 1)
+        {
+          // first check if there's a default at all...
+          SimpleXMLTransfer *presets;
+          presets = Global::aircraft->getFDMInterface()->getMixerPresets();
+          if (presets != NULL)
+          {
+            // o.k., take the values from the first preset
+            SimpleXMLTransfer *def_mixer = presets->getChildAt(0);
+            int err = Global::TXInterface->mixer->reinit(def_mixer);
+            if (err)
+            {
+              // error, restore mixer definition from cfgfile
+              Global::TXInterface->mixer->reinit(cfgfile, false);
+            }
+            // Global::TXInterface->mixer->putBackIntoCfg might be called now
           }
-          
         }
                 
-        initialize_flight_model();
         if (Global::soundserver != (CRRCAudioServer*)0)
           Global::soundserver->pause(false);
+        Global::Simulation->reset();
       }
       catch (std::runtime_error& e)
       {
