@@ -346,19 +346,13 @@ CGUIMixerDialog::CGUIMixerDialog(T_TX_Interface* itxi)
   comboPresets->activate();
   comboPresets->setUserData(this);
 
+  // finalize the dialog
   close();
   // keep largest width
   //setSize(3*DLG_DEF_SPACE + 4*SLIDER_W + 1*SLIDER_LABEL_W + 2*DLG_DEF_SPACE, y_in_win);
   setSize(2*DLG_DEF_SPACE + COMBO_LABEL_W + COMBO_W, y_in_win);
   setCallback(CGUIMixerCallback);
-
-  // center the dialog on screen
-  int wwidth, wheight;
-  int current_width = getABox()->max[0] - getABox()->min[0];
-  int current_height = getABox()->max[1] - getABox()->min[1];
-  puGetWindowSize(&wwidth, &wheight);
-  setPosition(wwidth/2 - current_width/2, wheight/2 - current_height/2);
-
+  centerOnScreen();
   reveal();
 }
 
@@ -581,6 +575,9 @@ void CGUIMixerCallback(puObject *obj)
       dlg->txi->mixer->mixer_val[i]     = dlg->slider_mix_val[i]->getFloatValue();
       dlg->txi->mixer->mixer_dst[i]     = dlg->combo_mix_dst[i]->getCurrentItem();    
     }
+    
+    // Save new settings
+    dlg->txi->mixer->putBackIntoCfg(cfgfile);
   }
   delete dlg->presetGrp;
   puDeleteObject(obj);
@@ -591,10 +588,9 @@ void CGUIMixerCallback(puObject *obj)
  */
 void CGUIMixerEnableButtonCallback(puObject *obj)
 {
-  puButton *btn = static_cast<puButton*>(obj);
-  CGUIMixerDialog* dlg = static_cast<CGUIMixerDialog*>(btn->getUserData());
+  CGUIMixerDialog* dlg   = (CGUIMixerDialog*)obj->getUserData();
   
-  if (btn->getIntegerValue())
+  if (obj->getIntegerValue())
   {
     dlg->all_widgets->reveal();
   }
