@@ -75,9 +75,9 @@ CGUIWindThermalDialog::CGUIWindThermalDialog()
 
   // Create widgets
   //
-  comboPresetsWind = new puaComboBox(LABEL_W + DLG_DEF_SPACE, BUTTON_BOX_HEIGHT + 12* DLG_DEF_SPACE + 11* SLIDER_H + 1*COMBO_H,
-                                    LABEL_W + COMBO_W,       BUTTON_BOX_HEIGHT + 12* DLG_DEF_SPACE + 11* SLIDER_H + 2*COMBO_H,
-                                    NULL, false);
+  comboPresetsWind = new puaComboBox(LABEL_W + DLG_DEF_SPACE, BUTTON_BOX_HEIGHT + 13* DLG_DEF_SPACE + 12* SLIDER_H + 1*COMBO_H,
+                                     LABEL_W + COMBO_W,       BUTTON_BOX_HEIGHT + 13* DLG_DEF_SPACE + 12* SLIDER_H + 2*COMBO_H,
+                                     NULL, false);
   comboPresetsWind->setChildColourScheme(PUCLASS_POPUPMENU, dlgCol1[0], dlgCol1[1], dlgCol1[2]);
   comboPresetsWind->newList(presetsWind);
   comboPresetsWind->setLabelPlace(PUPLACE_CENTERED_LEFT);
@@ -88,8 +88,8 @@ CGUIWindThermalDialog::CGUIWindThermalDialog()
   comboPresetsWind->activate();
   comboPresetsWind->setUserData(this);
 
-  slider_windVelocity = new crrcSlider(LABEL_W + DLG_DEF_SPACE,            BUTTON_BOX_HEIGHT + 11* DLG_DEF_SPACE + 10* SLIDER_H + 1*COMBO_H,
-                                       LABEL_W + DLG_DEF_SPACE + SLIDER_W, BUTTON_BOX_HEIGHT + 11* DLG_DEF_SPACE + 11* SLIDER_H + 1*COMBO_H,
+  slider_windVelocity = new crrcSlider(LABEL_W + DLG_DEF_SPACE,            BUTTON_BOX_HEIGHT + 12* DLG_DEF_SPACE + 11* SLIDER_H + 1*COMBO_H,
+                                       LABEL_W + DLG_DEF_SPACE + SLIDER_W, BUTTON_BOX_HEIGHT + 12* DLG_DEF_SPACE + 12* SLIDER_H + 1*COMBO_H,
                                        NUM_W);
   slider_windVelocity->setLabelPlace(PUPLACE_CENTERED_LEFT);
   slider_windVelocity->setLabel(_("Velocity [ft/s]"));
@@ -99,8 +99,8 @@ CGUIWindThermalDialog::CGUIWindThermalDialog()
   slider_windVelocity->setStepSize(1);
   slider_windVelocity->setValue(cfg->wind->getVelocity());
 
-  slider_windDir = new crrcSlider(LABEL_W + DLG_DEF_SPACE,            BUTTON_BOX_HEIGHT + 10* DLG_DEF_SPACE +  9* SLIDER_H + 1*COMBO_H,
-                                  LABEL_W + DLG_DEF_SPACE + SLIDER_W, BUTTON_BOX_HEIGHT + 10* DLG_DEF_SPACE + 10* SLIDER_H + 1*COMBO_H,
+  slider_windDir = new crrcSlider(LABEL_W + DLG_DEF_SPACE,            BUTTON_BOX_HEIGHT + 11* DLG_DEF_SPACE + 10* SLIDER_H + 1*COMBO_H,
+                                  LABEL_W + DLG_DEF_SPACE + SLIDER_W, BUTTON_BOX_HEIGHT + 11* DLG_DEF_SPACE + 11* SLIDER_H + 1*COMBO_H,
                                   NUM_W);
   slider_windDir->setLabelPlace(PUPLACE_CENTERED_LEFT);
   slider_windDir->setLabel(_("Direction"));
@@ -111,6 +111,18 @@ CGUIWindThermalDialog::CGUIWindThermalDialog()
   slider_windDir->setValue(cfg->wind->getDirection());
   //if direction imposed on scenery file description invalidate wind slider :
   if( Global::scenery->getImposeWindDirection() ) slider_windDir->greyOut();
+
+  // Use turbulence model check box
+  slider_windTurbulence = new crrcSlider(LABEL_W + DLG_DEF_SPACE,            BUTTON_BOX_HEIGHT + 10* DLG_DEF_SPACE +  9* SLIDER_H + 1*COMBO_H,
+                                         LABEL_W + DLG_DEF_SPACE + SLIDER_W, BUTTON_BOX_HEIGHT + 10* DLG_DEF_SPACE + 10* SLIDER_H + 1*COMBO_H,
+                                         NUM_W);
+  slider_windTurbulence->setLabelPlace(PUPLACE_CENTERED_LEFT);
+  slider_windTurbulence->setLabel(_("Turbulence intensity [rel]"));
+  slider_windTurbulence->setSliderFraction(0.05);
+  slider_windTurbulence->setMinValue(0);
+  slider_windTurbulence->setMaxValue(2);
+  slider_windTurbulence->setStepSize(0.2);
+  slider_windTurbulence->setValue(cfg->wind->getTurbulence());
   
   inputNewWind = new puInput(DLG_DEF_SPACE,           BUTTON_BOX_HEIGHT + 9* DLG_DEF_SPACE + 8* SLIDER_H + 1*COMBO_H,
                              LABEL_W + DLG_DEF_SPACE, BUTTON_BOX_HEIGHT + 9* DLG_DEF_SPACE + 9* SLIDER_H + 1*COMBO_H);
@@ -222,18 +234,12 @@ CGUIWindThermalDialog::CGUIWindThermalDialog()
   buttonTmp->setCallback(CGUIThermalNewPresetCallback);
   buttonTmp->setUserData(this);          
       
+  // finalize the dialog
   close();
   setSize(SLIDER_W + LABEL_W + 2*DLG_DEF_SPACE,
-          11*SLIDER_H + 13*DLG_DEF_SPACE + 2*COMBO_H + BUTTON_BOX_HEIGHT);
+          12*SLIDER_H + 14*DLG_DEF_SPACE + 2*COMBO_H + BUTTON_BOX_HEIGHT);
   setCallback(CGUIWindThermalCallback);
-
-  // center the dialog on screen
-  int wwidth, wheight;
-  int current_width = getABox()->max[0] - getABox()->min[0];
-  int current_height = getABox()->max[1] - getABox()->min[1];
-  puGetWindowSize(&wwidth, &wheight);
-  setPosition(wwidth/2 - current_width/2, wheight/2 - current_height/2);
-
+  centerOnScreen();
   reveal();
 }
 
@@ -259,12 +265,14 @@ void CGUIWindPresetCallback(puObject *obj)
   {
     dlg->slider_windDir->setValue(270);
     dlg->slider_windVelocity->setValue(5);
+    dlg->slider_windTurbulence->setValue(1);
   }
   else
   {
     SimpleXMLTransfer* preset = dlg->presetGrpWind->getChildAt(nItem-1);
-    dlg->slider_windDir->setValue((float)preset->getDouble("direction"));
-    dlg->slider_windVelocity->setValue((float)preset->getDouble("velocity"));
+    dlg->slider_windDir->setValue((float)preset->getDouble("direction",270.));
+    dlg->slider_windVelocity->setValue((float)preset->getDouble("velocity",5.));
+    dlg->slider_windTurbulence->setValue((float)preset->getDouble("turbulence",1.));
   }
 }
 
@@ -275,12 +283,13 @@ void CGUIWindNewPresetCallback(puObject *obj)
 {
   CGUIWindThermalDialog* dlg   = (CGUIWindThermalDialog*)obj->getUserData();
   
-  SimpleXMLTransfer* wind    = new SimpleXMLTransfer();
+  SimpleXMLTransfer* wind      = new SimpleXMLTransfer();
   
   wind->setName("wind");
-  wind->setAttribute("name_en",   dlg->inputNewWind->getStringValue());
-  wind->setAttribute("velocity",  dlg->slider_windVelocity->getStringValue());
-  wind->setAttribute("direction", dlg->slider_windDir->getStringValue());
+  wind->setAttribute("name_en",    dlg->inputNewWind->getStringValue());
+  wind->setAttribute("velocity",   dlg->slider_windVelocity->getStringValue());
+  wind->setAttribute("direction",  dlg->slider_windDir->getStringValue());
+  wind->setAttribute("turbulence", dlg->slider_windTurbulence->getStringValue());
   
   cfgfile->getChild("presets.wind", true)->addChild(wind);
   
@@ -388,6 +397,7 @@ void CGUIWindThermalCallback(puObject *obj)
     
     cfg->wind->setDirection(dlg->slider_windDir->getFloatValue(), cfg);
     cfg->wind->setVelocity(dlg->slider_windVelocity->getFloatValue());
+    cfg->wind->setTurbulence(dlg->slider_windTurbulence->getFloatValue());
         
     SimpleXMLTransfer* thermaldata = cfg->getCurLocCfgPtr(cfgfile)->getChild("thermal");
     thermaldata->setAttributeOverwrite("strength_mean",  dlg->slider_thermalStrengthMean->getStringValue());
