@@ -170,8 +170,9 @@ int T_Thermal::putBackIntoCfg(SimpleXMLTransfer* cfgfile)
 
 void T_Wind::setDefaults()
 {
-  velocity  =   7;
-  direction = 270;
+  velocity   = 7;
+  direction  = 270;
+  turbulence = 1; // turbulence on, standard intensity
 }
 
 void T_Wind::setVelocity(double val)
@@ -190,6 +191,11 @@ void T_Wind::setDirection(double val, T_Config* cfg)
   }
 }
 
+void T_Wind::setTurbulence(double val)
+{
+  turbulence = val;  
+}
+
 T_Wind::T_Wind()
 {
   setDefaults();
@@ -209,6 +215,7 @@ void T_Wind::read(SimpleXMLTransfer* cfgfile, T_Config* cfg)
   // the global defaults
   float flDirection = 270;
   float flSpeed = 7;
+  float wdTurbulence = 1;
   int ImposeDirection = false;
   
   // try to override defaults with values from the current scenery object
@@ -217,7 +224,9 @@ void T_Wind::read(SimpleXMLTransfer* cfgfile, T_Config* cfg)
     flDirection = Global::scenery->getDefaultWindDirection();
     flSpeed = Global::scenery->getDefaultWindSpeed();
     ImposeDirection = Global::scenery->getImposeWindDirection();
-    printf("Default wind from scenery: %.2f ft/s at %.2f deg\n", flSpeed, flDirection);
+    wdTurbulence = Global::scenery->getDefaultTurbulence();
+    printf("Default wind from scenery: %.2f ft/s at %.2f deg, turbulence (rel.) %.1f\n",
+           flSpeed, flDirection, wdTurbulence);
   }
   
   // now try to override these defaults with values from the config file
@@ -225,25 +234,29 @@ void T_Wind::read(SimpleXMLTransfer* cfgfile, T_Config* cfg)
   {
     SimpleXMLTransfer* el = cfg->getCurLocCfgPtr(cfgfile)->getChild("wind");
     if(!ImposeDirection) flDirection = el->getDouble("direction");
-    flSpeed     = el->getDouble("velocity");
+    flSpeed = el->getDouble("velocity");
+    wdTurbulence = el->getDouble("turbulence");
   }
   catch (XMLException e)
   {
     // nothing in the config file...
   }
-  setVelocity (flSpeed);
+  setVelocity(flSpeed);
   setDirection(flDirection, cfg);
+  setTurbulence(wdTurbulence);
     
   // show values:
-  printf("Wind: velocity=%f ft/sec   direction=%f\n",  velocity, direction);
+  printf("Wind: %.2f ft/s at %.2f deg, turbulence (rel.) %.1f\n",
+         velocity, direction, turbulence);
 }
 
 int T_Wind::putBackIntoCfg(SimpleXMLTransfer* cfgfile)
 {
   SimpleXMLTransfer* el = cfg->getCurLocCfgPtr(cfgfile)->getChild("wind", true);
 
-  el->setAttributeOverwrite("velocity",  doubleToString(velocity));
+  el->setAttributeOverwrite("velocity", doubleToString(velocity));
   el->setAttributeOverwrite("direction", doubleToString(direction));
+  el->setAttributeOverwrite("turbulence", doubleToString(turbulence));
 
   return(0);
 }
@@ -497,6 +510,7 @@ void T_Config::createDefaultConfig(SimpleXMLTransfer*& acfgfile)
   tex->addAttribute("name_en",      "Dynamic soaring Cape Cod");
   tex->addAttribute("velocity",     "13");
   tex->addAttribute("direction",    "90");
+  tex->addAttribute("turbulence",    "1");
   acfgfile->getChild("presets.wind", true)->addChild(tex);
   
   tex = new SimpleXMLTransfer();
@@ -504,6 +518,7 @@ void T_Config::createDefaultConfig(SimpleXMLTransfer*& acfgfile)
   tex->addAttribute("name_en",      "Soaring slow Cape Cod");
   tex->addAttribute("velocity",     "10");
   tex->addAttribute("direction",    "270");
+  tex->addAttribute("turbulence",   "1");
   acfgfile->getChild("presets.wind")->addChild(tex);
   
   tex = new SimpleXMLTransfer();
@@ -511,6 +526,7 @@ void T_Config::createDefaultConfig(SimpleXMLTransfer*& acfgfile)
   tex->addAttribute("name_en",      "Soaring ruff Cape Cod");
   tex->addAttribute("velocity",     "18");
   tex->addAttribute("direction",    "270");
+  tex->addAttribute("turbulence",   "1");
   acfgfile->getChild("presets.wind")->addChild(tex);
   
   tex = new SimpleXMLTransfer();
@@ -518,6 +534,7 @@ void T_Config::createDefaultConfig(SimpleXMLTransfer*& acfgfile)
   tex->addAttribute("name_en",      "F3F competition at Cape Cod");
   tex->addAttribute("velocity",     "30");
   tex->addAttribute("direction",    "270");
+  tex->addAttribute("turbulence",   "1");
   acfgfile->getChild("presets.wind")->addChild(tex);
   
   tex = new SimpleXMLTransfer();
@@ -525,6 +542,7 @@ void T_Config::createDefaultConfig(SimpleXMLTransfer*& acfgfile)
   tex->addAttribute("name_en",      "Heli (no wind)");
   tex->addAttribute("velocity",     "0");
   tex->addAttribute("direction",    "180");
+  tex->addAttribute("turbulence",   "0");
   acfgfile->getChild("presets.wind")->addChild(tex);
 }
 
