@@ -69,6 +69,7 @@ If you'd like to help with CRRCSIM, then send me an email!
 #include "zoom.h"
 #include "CTime.h"
 #include "mod_mode/T_GameHandler.h"
+#include "mod_mode/F3A/handlerF3A.h"
 #include "mod_mode/F3F/handlerF3F.h"
 #include "mod_fdm/formats/airtoxml.h"
 #include "mod_fdm/xmlmodelfile.h"
@@ -101,6 +102,7 @@ If you'd like to help with CRRCSIM, then send me an email!
 
 #include "record.h"
 #include "robots.h"
+#include "mod_video/fonts.h"
 
 
 #include <math.h>
@@ -113,10 +115,10 @@ If you'd like to help with CRRCSIM, then send me an email!
 //Configuration Data
 //TInputDev  inputDev;
 
-CTime   *crrc_time;
+// Game timing control
+CTime *crrc_time;
 
-
-// Pointer to scenery object and sky sphere
+// Player position in FDM coordinates
 CRRCMath::Vector3 player_pos;
 
 // Variometer sound
@@ -130,44 +132,32 @@ FDMEnviroment* fdmenv = 0;
 /*****************************************************************************/
 void activate_test_mode()
 {
-  double dDist;
-
-  TSimInputs inp  = TSimInputs();
+  TSimInputs inp = TSimInputs();
   
-  Global::testmode.test_mode  = TRUE;
+  Global::testmode = TRUE;
   
-  // Set distance to plane to make it visible in a good size, set 
-  // fixed flFieldOfView.
-  {            
-    double dAircraftSize;
-    float  flFieldOfView; // degrees
-   
-    Global::testmode.flAutozoom = flAutozoom;
-    flAutozoom          = 0; // autozoom disabled
-    
-    dAircraftSize = Global::aircraft->getFDM()->getAircraftSize();
-    // this is just to read flFieldOfView
-    flFieldOfView = zoom_calc(dAircraftSize);
-    
-    // 1 * 2* tan(M_PI * flFieldOfView /2 / 180) * dDist  = 2*dAircraftSize
-    dDist = dAircraftSize / ( 1 * tan(M_PI * flFieldOfView / 2 / 180) );
-  }
+  // Set distance to plane to make it visible in a good size, 
+  // at an average field of view of 35deg:  
+  // tan(fieldOfView/2) * distance = aircraftSize
+  // with aircraftSize approximately equal to half model size.
+  // Take some 30% margin to allow for model movement.
+  float fieldOfView = 35.0;
+  float aircraftSize = Global::aircraft->getModel()->getAircraftSize();
+  float distance = 1.3*aircraftSize/tan(M_PI/180.0*0.5*fieldOfView);
   
   // enter testmode
   CRRCMath::Vector3 ppos = Global::scenery->getPlayerPosition();
-  CRRCMath::Vector3 planeposn(ppos.r[0] - dDist, ppos.r[1], ppos.r[2]);
-  Global::aircraft->enterTestmode(planeposn); 
+  CRRCMath::Vector3 planepos(ppos.r[0] - distance, ppos.r[1], ppos.r[2]);
+  Global::aircraft->enterTestmode(planepos); 
   initialize_flight_model();
   Global::aircraft->getFDMInterface()->update(&inp, 0, 0);
-  Video::UpdateCamera(1);//Only once here to initialize. Then, the camera is still
- 
+  Video::UpdateCamera(1); //Only once here to initialize. Then, the camera is still
 }
 
 void leave_test_mode()
 {
-  Global::testmode.test_mode  = FALSE;
+  Global::testmode = FALSE;
   Global::aircraft->leaveTestmode();
-  flAutozoom = Global::testmode.flAutozoom;
 }
 
 /*****************************************************************************/
@@ -286,26 +276,6 @@ void Init_mod_windfield()
 }
 
 /*****************************************************************************/
-void calc_fps()
-{
-  static int last_time = 0;
-  int cur_time, time_diff;
-  static int avg_diff = 10;
-  int avg_fps;
-
-  cur_time = SDL_GetTicks();
-  time_diff = (cur_time - last_time);
-
-  if (time_diff < 1)
-    time_diff = 10;
-
-  avg_diff += ((time_diff << 8) - avg_diff) >> 5;
-
-  avg_fps = (int)rintf(1000.0 / (avg_diff >> 8));
-  Global::nFPS = avg_fps;
-
-  last_time = cur_time;
-}
 
 /**
  * Reverts to mouse input.
@@ -475,8 +445,16 @@ void read_config_into_globals()
   Global::wind_mode = cfgfile->getInt("wind_mode.fUse", 2);
   Global::nVerbosity = cfgfile->getInt("nVerbosity.level", 0);
   Global::HUDCompass = cfgfile->getInt("HUDCompass.fUse", 0);
+  Global::windVectors = cfgfile->getInt("windVectors.fUse", 0);
+  Global::modelView = cfgfile->getInt("modelViewWindow.fUse", 2);
   Global::dt = cfgfile->getDouble("simulation.flightModel.dt", 0.002777);
+  Global::slowMotion = cfgfile->getInt("simulation.slowMotion", 0);
+  Global::slowTimeScale = cfgfile->getDouble("simulation.slowTimeScale", 2.0);
   Video::read_config(cfgfile);
+  
+  // round dt to the integer millisecond, as this allow for
+  // maximum display smoothness (see CTime.cpp)
+  Global::dt = (int)(Global::dt*1000 + 0.5)/1000.;
 }
 
 void write_globals_into_config()
@@ -485,8 +463,13 @@ void write_globals_into_config()
   cfgfile->setAttributeOverwrite("wind_mode.fUse", Global::wind_mode);
   cfgfile->setAttributeOverwrite("nVerbosity.level", Global::nVerbosity);
   cfgfile->setAttributeOverwrite("HUDCompass.fUse", Global::HUDCompass);
+  cfgfile->setAttributeOverwrite("windVectors.fUse", Global::windVectors);
+  cfgfile->setAttributeOverwrite("modelViewWindow.fUse", Global::modelView);
   cfgfile->setAttributeOverwrite("simulation.flightModel.dt",
                                  doubleToString(Global::dt));
+  cfgfile->setAttributeOverwrite("simulation.slowMotion", Global::slowMotion);
+  cfgfile->setAttributeOverwrite("simulation.slowTimeScale",
+                                 doubleToString(Global::slowTimeScale));
 }
 
 /*****************************************************************************/
@@ -624,17 +607,25 @@ int main(int argc,char **argv)
   
   try
   {
-    //Internationalisation
+    // Internationalisation
     char *loc = setlocale (LC_MESSAGES, "");
-    printf("Locale : %s\n",loc);
+    // getDataPath only looks for file not folder, so the following does not work
+    //string locale_dir =  FileSysTools::getDataPath("locale");
+    //printf("dir_locale: %s\n", locale_dir.c_str());
+    //bindtextdomain ("crrcsim",locale_dir.c_str());
     bindtextdomain ("crrcsim","./locale/");
     textdomain ("crrcsim");
-    bind_textdomain_codeset ("crrcsim","iso-8859-15");//Plib does not work with UTF-8
+    //Plib does not work with UTF-8. It is necessary to make a 8bits transcoding 
+    bind_textdomain_codeset ("crrcsim","iso8859-15");//Temporarily, before obtaining the correct codeset
+    string codeset = _("iso8859-15");//return correct codeset
+    bind_textdomain_codeset ("crrcsim",codeset.c_str());
+    printf("Locale : %s Internal codeset: %s\n",loc, codeset.c_str());
+    Video::codeSet = codeset;
     Global::TXInterface = (T_TX_Interface*)0;
 
     Global::inputDev = new TInputDev();
     FileSysTools::SetAppname("crrcsim");
-    Global::testmode.test_mode = FALSE;
+    Global::testmode = FALSE;
 
     Global::Simulation = new SimStateHandler();
     
@@ -705,7 +696,7 @@ int main(int argc,char **argv)
           crrc_exit(CRRC_EXIT_FAILURE);
 
         // must be after crrc_checkopts because crrc_checkopts can change
-        //   video.enabled and sound.enabled based on command line options
+        // video.enabled and sound.enabled based on command line options
         if (cfgfile->getInt("video.enabled", 1))
           SDLFlags |= SDL_INIT_VIDEO;
         if (cfgfile->getInt("sound.enabled", 1))
@@ -713,8 +704,7 @@ int main(int argc,char **argv)
         SDL_Init(SDLFlags);
         SDL_EnableUNICODE(1); // We need this to pass keys to pui
         SDL_EnableKeyRepeat(50, 150);
-        
-        
+                
         Global::recorder = new FlightRecorder(FileSysTools::getHomePath());
         Global::robots = new Robots();
         
@@ -728,12 +718,9 @@ int main(int argc,char **argv)
         if (cfgfile->getInt("video.enabled", 1))
         {
           Video::setupScreen(0, 0, 0);
+          Video::setWindowTitleString();
         }
         
-        // ***** Setting window caption *****************************************
-        if (cfgfile->getInt("video.enabled", 1))
-          Video::setWindowTitleString();
-
         // ***** Sound **********************************************************
         if (cfgfile->getInt("sound.enabled", 1))
         {
@@ -761,9 +748,10 @@ int main(int argc,char **argv)
         // ***** Video **********************************************************
 
         Video::initialize_scenegraph();
-        Global::scenery  = new SceneryNull();/*Temporarily an empty scenery. 
-                    It allows to have a graphic context to display a message
-                    during the load of the configured scenery.*/
+        // Temporarily empty scenery. 
+        // It allows to have a graphic context to display a message
+        // during the load of the configured scenery
+        Global::scenery = new SceneryNull();
 
         Init_mod_windfield();
 
@@ -848,12 +836,19 @@ int main(int argc,char **argv)
     player_pos = Global::scenery->getPlayerPosition();
     Init_mod_windfield();
     
+    //initialise f3a game mode
+    if (cfgfile->getInt("game.f3a.enabled",0))
+    {
+      delete Global::gameHandler;
+      Global::gameHandler= new HandlerF3A();
+    }  
+        
     //initialise f3f game mode
-     if (cfgfile->getInt("game.f3f.enabled",0))
-     {
-       delete Global::gameHandler;
-       Global::gameHandler= new HandlerF3F();
-     }  
+    if (cfgfile->getInt("game.f3f.enabled",0))
+    {
+      delete Global::gameHandler;
+      Global::gameHandler= new HandlerF3F();
+    }  
         
     Global::Simulation->reset();
     
@@ -862,7 +857,6 @@ int main(int argc,char **argv)
     
     while (Global::Simulation->getState() != STATE_EXIT)
     {
-      crrc_time->update();
       scheduler.Run();
 
       Global::TXInterface->getInputData(&Global::inputs);
@@ -877,7 +871,7 @@ int main(int argc,char **argv)
         Global::inputs.heli_fixed_z = EOM01_FIXED_Z_OFF;
       }
 
-      Global::Simulation->doIdle(&Global::inputs);
+      Global::Simulation->doIdle(&Global::inputs, crrc_time->update());
 
       Global::inputs.ClearKeys();
       
@@ -888,18 +882,14 @@ int main(int argc,char **argv)
       }
 
       // get aircraft position from FDM
-      CRRCMath::Vector3 vFdmPos = Global::aircraft->getPos();
-      CRRCMath::Vector3 vAircraftPos(     vFdmPos.r[0],
-                                     -1 * vFdmPos.r[2],
-                                          vFdmPos.r[1]);
-      float  distance_to_model = (vAircraftPos - player_pos).length();
+      CRRCMath::Vector3 aircraft_pos = Video::FDM2Graphics(Global::aircraft->getPos());
+      float distance_to_model = (aircraft_pos - player_pos).length();
       
       field_of_view = zoom_calc(distance_to_model);
       if (Global::gui)
       {
         Video::adjust_zoom(field_of_view);
       }
-      calc_fps();
       
       #if 0
       Global::verboseString += " X: " + ftoStr(vFdmPos.r[0], 2, 2, true, false);
@@ -914,7 +904,7 @@ int main(int argc,char **argv)
       switch (Global::nVerbosity)
       {
        case 3:
-        Global::verboseString += "FPS: " + itoStr(Global::nFPS, ' ', 1) + " ";
+        Global::verboseString += "FPS: " + itoStr(crrc_time->getGameFPS(), ' ', 1) + " ";
         //fallthrough
        case 2:
         Global::verboseString += "FoV: " + ftoStr(field_of_view, 2, 1, false, false);
@@ -980,7 +970,7 @@ int main(int argc,char **argv)
 
       if (Global::gui)
       {
-          Global::gui->doHUDCompass(field_of_view);
+        Global::gui->doHUDCompass(field_of_view);
       }
       
       if (Global::gui)
@@ -998,7 +988,7 @@ int main(int argc,char **argv)
       {
         soundUpdate3D(distance_to_model,
                       Global::aircraft->getFDM()->getPropFreq(),
-                      -1*vFdmPos.r[2],
+                      aircraft_pos.r[1],
                       Global::aircraft->getFDM()->getVRelAirmass()/Global::aircraft->getFDM()->getTrimmedFlightVelocity());
       }
     }
