@@ -136,11 +136,11 @@ void CRRC_AirplaneSim_002::update(TSimInputs* inputs,
   double V_north_zm,V_east_zm,V_down_zm;
 
   /**
-   * Using a length of about roughly one half of the aircrafts
-   * size to calculate wind gradients. 0.1 foot had been used before,
+   * Using a length of about roughly one half of the aircraft's
+   * span to calculate wind gradients. 0.1 foot had been used before,
    * which leads to very high or zero gradients.
    */
-  double delta_space = getAircraftSize()/2;
+  double delta_space = 0.5*getWingspan();
   
   // Anpassung an LaRCSim: dort ist H=-Z
   CRRCMath::Vector3 v_P_CG_Rwy = eom.pos.val;
