@@ -91,7 +91,7 @@ void CRRC_AirplaneSim_Heli01::update(TSimInputs* inputs,
                                      int         multiloop) 
 {
   CRRCMath::Vector3 v_V_local_airmass;
-  CRRCMath::Vector3 v_V_gust_local = CRRCMath::Vector3();
+  CRRCMath::Vector3 v_V_gust_body = CRRCMath::Vector3();
   
   env->CalculateWind(v_P_CG_Rwy.r[0],        v_P_CG_Rwy.r[1],        v_P_CG_Rwy.r[2],
                      v_V_local_airmass.r[0], v_V_local_airmass.r[1], v_V_local_airmass.r[2]);
@@ -113,7 +113,7 @@ void CRRC_AirplaneSim_Heli01::update(TSimInputs* inputs,
 #endif
             
     ls_step( dt );
-    ls_aux(v_V_local_airmass, v_V_gust_local);
+    ls_aux(v_V_local_airmass, v_V_gust_body);
 
     env->ControllerCallback(dt, this, inputs, &myInputs);
     
