@@ -99,66 +99,33 @@ void CRRC_AirplaneSim_Larcsim::update(TSimInputs* inputs,
                                       double      dt,
                                       int         multiloop) 
 {
-  double V_north_xp,V_east_xp,V_down_xp;   // temp vars for calculating wind grad.
-  double V_north_yp,V_east_yp,V_down_yp;
-  double V_north_zp,V_east_zp,V_down_zp;
-  double V_north_xm,V_east_xm,V_down_xm;
-  double V_north_ym,V_east_ym,V_down_ym;
-  double V_north_zm,V_east_zm,V_down_zm;
-  
-  /**
-   * Gradients of wind velocity, runway coordinates.
-   * m_V_atmo_rwy.v[0][?] is U_atmo_?
-   * m_V_atmo_rwy.v[1][?] is V_atmo_?
-   * m_V_atmo_rwy.v[2][?] is W_atmo_?
-   */
-  CRRCMath::Matrix33 m_V_atmo_rwy;
-  
   CRRCMath::Vector3 v_V_local_airmass;
-  CRRCMath::Vector3 v_V_gust_local = CRRCMath::Vector3();
+  CRRCMath::Vector3 v_V_gust_body, v_R_omega_gust_body;
+  CRRCMath::Matrix33 m_V_local_airmass_grad;  
 
+  CRRCMath::Vector3 v_F_aero, v_F_engine, v_F_gear; // Force x/y/z
+  CRRCMath::Vector3 v_M_aero, v_M_engine, v_M_gear; // l/m/n <-> roll/pitch/yaw
+
+  int nAircraftOutsideWindfieldSim = 
+    env->CalculateWind(v_P_CG_Rwy.r[0],        v_P_CG_Rwy.r[1],        v_P_CG_Rwy.r[2],
+                       v_V_local_airmass.r[0], v_V_local_airmass.r[1], v_V_local_airmass.r[2]);
+                       
   /**
-   * Using a length of about roughly one half of the aircrafts
-   * size to calculate wind gradients. 0.1 foot had been used before,
+   * Using a length of about roughly one half of the aircraft's
+   * span to calculate wind gradients. 0.1 foot had been used before,
    * which leads to very high or zero gradients.
    */
-  double delta_space = getAircraftSize()/2;
+  double delta_space = 0.5*getWingspan();
   
-  int   nAircraftOutsideWindfieldSim = 
-    env->CalculateWind(v_P_CG_Rwy.r[0]+delta_space, v_P_CG_Rwy.r[1],             v_P_CG_Rwy.r[2],
-                       V_north_xp,                  V_east_xp,                   V_down_xp) |
-    env->CalculateWind(v_P_CG_Rwy.r[0],             v_P_CG_Rwy.r[1]+delta_space, v_P_CG_Rwy.r[2],
-                       V_north_yp,                  V_east_yp,                   V_down_yp) |
-    env->CalculateWind(v_P_CG_Rwy.r[0],             v_P_CG_Rwy.r[1],             v_P_CG_Rwy.r[2]+delta_space,
-                       V_north_zp,                  V_east_zp,                   V_down_zp) |
-    env->CalculateWind(v_P_CG_Rwy.r[0]-delta_space, v_P_CG_Rwy.r[1],             v_P_CG_Rwy.r[2],
-                       V_north_xm,                  V_east_xm,                   V_down_xm) |
-    env->CalculateWind(v_P_CG_Rwy.r[0],             v_P_CG_Rwy.r[1]-delta_space, v_P_CG_Rwy.r[2],
-                       V_north_ym,                  V_east_ym,                   V_down_ym) |
-    env->CalculateWind(v_P_CG_Rwy.r[0],             v_P_CG_Rwy.r[1],             v_P_CG_Rwy.r[2]-delta_space,
-                       V_north_zm,                  V_east_zm,                   V_down_zm) |
-    env->CalculateWind(v_P_CG_Rwy.r[0],             v_P_CG_Rwy.r[1],             v_P_CG_Rwy.r[2],
-                       v_V_local_airmass.r[0],      v_V_local_airmass.r[1],      v_V_local_airmass.r[2]);  
+  nAircraftOutsideWindfieldSim |= 
+    env->CalculateWindGrad(v_P_CG_Rwy.r[0], v_P_CG_Rwy.r[1], v_P_CG_Rwy.r[2], delta_space,
+                           m_V_local_airmass_grad);
 
   if (nAircraftOutsideWindfieldSim)
   {
     env->AddLogMsg("Error: aircraft outside windfield simulation");
   }
-  
-  // Gradients are calculated from symmetric pairs to get symmetric behaviour.
-  m_V_atmo_rwy.v[0][0] = (V_north_xp - V_north_xm)/(2*delta_space);
-  m_V_atmo_rwy.v[0][1] = (V_north_yp - V_north_ym)/(2*delta_space);
-  m_V_atmo_rwy.v[0][2] = (V_north_zp - V_north_zm)/(2*delta_space);
-  m_V_atmo_rwy.v[1][0] = (V_east_xp  - V_east_xm) /(2*delta_space);
-  m_V_atmo_rwy.v[1][1] = (V_east_yp  - V_east_ym) /(2*delta_space);
-  m_V_atmo_rwy.v[1][2] = (V_east_zp  - V_east_zm) /(2*delta_space);
-  m_V_atmo_rwy.v[2][0] = (V_down_xp  - V_down_xm) /(2*delta_space);
-  m_V_atmo_rwy.v[2][1] = (V_down_yp  - V_down_ym) /(2*delta_space);
-  m_V_atmo_rwy.v[2][2] = (V_down_zp  - V_down_zm) /(2*delta_space);
-  
-  CRRCMath::Vector3 v_F_aero, v_F_engine, v_F_gear; // Force x/y/z
-  CRRCMath::Vector3 v_M_aero, v_M_engine, v_M_gear; // l/m/n <-> roll/pitch/yaw
-  
+    
   for (int n=0; n<multiloop; n++)
   {
     logNewline();
@@ -176,12 +143,18 @@ void CRRC_AirplaneSim_Larcsim::update(TSimInputs* inputs,
     logVal(m_V_atmo_rwy);
 #endif
             
-    ls_step( dt );
-    ls_aux(v_V_local_airmass, v_V_gust_local);
+    ls_step(dt);
+
+    // update wind turbulence linear & rotational velocities
+    env->CalculateWindGust(dt, getAlt(), v_V_local_rel_airmass.length(), getWingspan(),
+                           v_V_local_airmass, LocalToBody,
+                           v_V_gust_body, v_R_omega_gust_body);
+    
+    ls_aux(v_V_local_airmass, v_V_gust_body);
 
     env->ControllerCallback(dt, this, inputs, &myInputs);
     
-    aero(&myInputs, m_V_atmo_rwy, v_F_aero, v_M_aero);
+    aero(&myInputs, m_V_local_airmass_grad, v_R_omega_gust_body, v_F_aero, v_M_aero);
     
 #if FDM_LOG_AERO_OUT != 0
     logVal(v_F_aero);
@@ -191,8 +164,12 @@ void CRRC_AirplaneSim_Larcsim::update(TSimInputs* inputs,
     engine(dt, &myInputs, v_F_engine, v_M_engine);
     gear(&myInputs, v_F_gear, v_M_gear);
 
-    /* Sum forces and moments at reference point (center of gravity) */
-    ls_accel(v_F_aero + v_F_engine + v_F_gear, v_M_aero + v_M_engine*effectivePropellerTorqueFactor + v_M_gear);
+    // Only a fraction of the total rolling torque is not cancelled
+    // by the aero effect of the prop wash on fuselage, wing and tail
+    v_M_engine.r[0] *= effectivePropellerTorqueFactor;
+    
+    // Sum forces and moments at reference point (center of gravity)
+    ls_accel(v_F_aero + v_F_engine + v_F_gear, v_M_aero + v_M_engine + v_M_gear);
   }
 }
 
@@ -307,17 +284,19 @@ void CRRC_AirplaneSim_Larcsim::LoadFromXML(SimpleXMLTransfer* xml, int nVerbosit
   Cn_dr = i->getDouble("Cn_dr");
   Cn_da = i->getDouble("Cn_da");
     
-  flap_drag   = aero->getDouble("flap.drag", 0);
-  flap_lift   = aero->getDouble("flap.lift", 0);
-  flap_moment = aero->getDouble("flap.moment", 0);
+  flap_drag      = aero->getDouble("flap.drag", 0);
+  flap_lift      = aero->getDouble("flap.lift", 0);
+  flap_moment    = aero->getDouble("flap.moment", 0);
+  flap_eff_ratio = aero->getDouble("flap.eff_ratio", 1.0);
   
-  spoiler_drag = aero->getDouble("spoiler.drag", 0);
-  spoiler_lift = aero->getDouble("spoiler.lift", 0);
+  spoiler_drag   = aero->getDouble("spoiler.drag", 0);
+  spoiler_lift   = aero->getDouble("spoiler.lift", 0);
+  spoiler_moment = aero->getDouble("spoiler.moment", 0);
   
   retract_drag = aero->getDouble("retract.drag", 0);
   retract_lift = aero->getDouble("retract.lift", 0);
   
-  effectivePropellerTorqueFactor = aero->getDouble("prop.torque.factor", 0.25);
+  effectivePropellerTorqueFactor = aero->getDouble("prop.torquefactor", 0.25);
   
   {
     double to_slug;
@@ -348,7 +327,6 @@ void CRRC_AirplaneSim_Larcsim::LoadFromXML(SimpleXMLTransfer* xml, int nVerbosit
   }
   
   wheels.init(xml, B_ref);
-  double span = wheels.getWingspan() * FT_TO_M;
 
   // calculate velocity in trimmed flight,
   // approximately since Gravity & Density are yet unknown.
@@ -399,7 +377,7 @@ void CRRC_AirplaneSim_Larcsim::LoadFromXML(SimpleXMLTransfer* xml, int nVerbosit
   if (nVerbosity > 1)
   {
     std::cout << "--- Airplane description: ---------------------------------------\n";
-    std::cout << "  Wingspan:                   " << span              << " m\n";
+    std::cout << "  Wingspan:                   " << (B_ref*FT_TO_M) << " m\n";
     std::cout << "  Mass:                       " << (Mass*SLUG_TO_KG) << " kg\n";
     std::cout << "  Velocity in trimmed flight: " << (trimmedFlightVelocity*FT_TO_M) << " m/s\n";
     std::cout << "-----------------------------------------------------------------\n";
@@ -456,18 +434,19 @@ void CRRC_AirplaneSim_Larcsim::engine( SCALAR dt, TSimInputs* inputs, CRRCMath::
  * Calculate forces and moments for the current time step.
  */
 void CRRC_AirplaneSim_Larcsim::aero(TSimInputs* inputs, 
-                                    CRRCMath::Matrix33& m_V_atmo_rwy,
+                                    CRRCMath::Matrix33 m_V_atmo_rwy,
+                                    CRRCMath::Vector3 v_R_omega_gust_body,
                                     CRRCMath::Vector3& v_F, CRRCMath::Vector3& v_M)
 {
   SCALAR elevator, aileron, rudder, flap, spoiler, gear_ext;
-
+  SCALAR flap_eff_factor;
   SCALAR Phat, Qhat, Rhat; // dimensionless rotation rates
   SCALAR CL_left, CL_cent, CL_right; // CL values across the span
   SCALAR dCL_left,dCL_cent,dCL_right; // stall-induced CL changes
   SCALAR dCD_left,dCD_cent,dCD_right; // stall-induced CD changes
   SCALAR dCl,dCn,dCl_stall,dCn_stall; // stall-induced roll,yaw moments
   SCALAR dCm_stall; // Stall induced pitching moment.
-  SCALAR CL_offset, CL_wing, CD_all, CD_scaled, Cl_w;
+  SCALAR CL_flap_offset, CL_offset, CL_wing, CD_all, CD_scaled, Cl_w;
   SCALAR Cl_r_mod,Cn_p_mod;
   SCALAR CL,CD,Cl,Cm,Cn;
   SCALAR QS;
@@ -476,7 +455,6 @@ void CRRC_AirplaneSim_Larcsim::aero(TSimInputs* inputs,
 
   CRRCMath::Vector3 v_R_omega_atmo;
   
-  CRRCMath::Matrix33 G;
   CRRCMath::Matrix33 m_V_body;
 
   SCALAR Cos_alpha, Sin_alpha, Cos_beta;
@@ -491,26 +469,28 @@ void CRRC_AirplaneSim_Larcsim::aero(TSimInputs* inputs,
   flap     = inputs->flap;
   spoiler  = inputs->spoiler;
   
+  /* To simulate the non linearity of flap effectiveness for large flap
+   * deflections (e.g. landing flap) a flap effectiveness ratio can be
+   * defined, giving the ratio R of flap derivatives at full deflection
+   * (i.e. flap = 0.5) vs at zero deflection. 
+   * R=1 for standard constant effectiveness (linear behaviour).
+   * The resulting flap effectiveness factor for the current deflection
+   * is then computed as k = 1 - (1 - R)*flap
+   */
+  flap_eff_factor = 1. - (1. - flap_eff_ratio)*flap; 
+   
   /* The retract input is 0.0 for fully extended gear, 1.0 for fully
    * retracted gear.
    */
   gear_ext = 1.0 - inputs->retract;
 
-  /* compute gradients of Local velocities w.r.t. Body coordinates
-      G_11  =  dU_local/dx_body
-      G_12  =  dU_local/dy_body   etc.  */
-    
-  G = m_V_atmo_rwy * LocalToBody.trans();
-
-  /* now compute gradients of Body velocities w.r.t. Body coordinates */
-  /*  U_body_x  =  dU_body/dx_body   etc.  */
-
-  m_V_body = LocalToBody * G;
+  /* transform wind velocity gradient from local to body frame */
+  m_V_body = LocalToBody * (m_V_atmo_rwy * LocalToBody.trans());
   
-  /* set rotation rates of airmass motion */
-  v_R_omega_atmo.r[0] =  m_V_body.v[2][0];
-  v_R_omega_atmo.r[1] = -m_V_body.v[2][1];
-  v_R_omega_atmo.r[2] =  m_V_body.v[1][2];
+  /* set rotation rates of airmass motion includin wind turbulence effect */
+  v_R_omega_atmo.r[0] =  m_V_body.v[2][1] + v_R_omega_gust_body.r[0];
+  v_R_omega_atmo.r[1] = -m_V_body.v[2][0] + v_R_omega_gust_body.r[1];
+  v_R_omega_atmo.r[2] =  m_V_body.v[1][0] + v_R_omega_gust_body.r[2];
 
   if (V_rel_wind != 0)
   {
@@ -528,9 +508,8 @@ void CRRC_AirplaneSim_Larcsim::aero(TSimInputs* inputs,
   }
 
   /* compute local CL at three spanwise locations */
-  CL_offset = flap_lift*flap
-            + spoiler_lift*spoiler
-            + retract_lift*gear_ext;
+  CL_flap_offset = flap_lift*flap_eff_factor*flap;
+  CL_offset = CL_flap_offset + spoiler_lift*spoiler + retract_lift*gear_ext;
   CL_left   = CL_0 + CL_a*(Alpha - Alpha_0 - Phat*eta_loc) + CL_offset;
   CL_cent   = CL_0 + CL_a*(Alpha - Alpha_0               ) + CL_offset;
   CL_right  = CL_0 + CL_a*(Alpha - Alpha_0 + Phat*eta_loc) + CL_offset;
@@ -549,8 +528,8 @@ void CRRC_AirplaneSim_Larcsim::aero(TSimInputs* inputs,
   // by half of the amount.  
   stalling=0;
   {
-    SCALAR CL_max_flap = CL_max + 0.5*CL_offset;
-    SCALAR CL_min_flap = CL_min + 0.5*CL_offset;
+    SCALAR CL_max_flap = CL_max + 0.5*CL_flap_offset;
+    SCALAR CL_min_flap = CL_min + 0.5*CL_flap_offset;
     
     if (CL_left  > CL_max_flap)
     {
@@ -592,14 +571,17 @@ void CRRC_AirplaneSim_Larcsim::aero(TSimInputs* inputs,
   CL = (CL_wing + CL_q*Qhat 
     + CL_de*elevator
     )*Cos_alpha;
-    
+  
+  /* save flight CL for public access */
+  flight_CL = CL;
+  
   /* correct profile CD for CL dependence and controls dependence */
   // Note quadratic effect of flap deflection
   CD_all = CD_prof
     + CD_CLsq*(CL_wing-CL_CD0)*(CL_wing-CL_CD0)
     + CD_AIsq*aileron*aileron
     + CD_ELsq*elevator*elevator
-    + flap_drag*flap*flap
+    + flap_drag*flap_eff_factor*flap*flap
     + spoiler_drag*spoiler
     + retract_drag*gear_ext;
   
@@ -610,7 +592,7 @@ void CRRC_AirplaneSim_Larcsim::aero(TSimInputs* inputs,
   }
   else
   {
-    CD_scaled=CD_all;
+    CD_scaled = CD_all;
   }
 
   /* Scale lateral cross-coupling derivatives with wing CL */
@@ -665,7 +647,8 @@ void CRRC_AirplaneSim_Larcsim::aero(TSimInputs* inputs,
   Cm = Cm_0 + Cm_a*(Alpha-Alpha_0) + Cm_q*Qhat 
     + Cm_de*elevator
     + dCm_stall
-    + flap_moment*flap*(1.0 - stalling);
+    + flap_moment*flap_eff_factor*flap*(1.0 - stalling)
+    + spoiler_moment*spoiler;
 
   /* set dimensional forces and moments */
   QS = 0.5*Density*V_rel_wind*V_rel_wind * S_ref;
@@ -696,8 +679,6 @@ void CRRC_AirplaneSim_Larcsim::ls_step_init()
   CRRCMath::Vector3 v_F_aero, v_F_engine, v_F_gear; // Force x/y/z
   CRRCMath::Vector3 v_M_aero, v_M_engine, v_M_gear; // l/m/n <-> roll/pitch/yaw
   
-  CRRCMath::Matrix33 dummy = CRRCMath::Matrix33();
-  
   TSimInputs ZeroInput = TSimInputs();
   
   EOM01::ls_step_init();
@@ -705,7 +686,7 @@ void CRRC_AirplaneSim_Larcsim::ls_step_init()
   /*    Initialize vehicle model                        */
   ls_aux(CRRCMath::Vector3(), CRRCMath::Vector3());
 
-  aero(&ZeroInput, dummy, v_F_aero, v_M_aero);
+  aero(&ZeroInput, CRRCMath::Matrix33(), CRRCMath::Vector3(), v_F_aero, v_M_aero);
   engine(0, &ZeroInput, v_F_engine, v_M_engine);
   gear(&ZeroInput, v_F_gear, v_M_gear);
 
