@@ -251,6 +251,15 @@ class DotAirLoader
       
 };
 
+void Check_fscanf(int nRead, int nExp)
+{
+  if (nRead < nExp)
+  {
+    std::string msg = "Error reading airplane specification file ";
+    throw std::runtime_error(msg);
+  }
+}
+
 DotAirLoader::DotAirLoader(const char* filename) /*{{{*/
 {
   FILE *airplane_data;
@@ -341,7 +350,8 @@ DotAirLoader::DotAirLoader(const char* filename) /*{{{*/
     while (fscanf(airplane_data,"%s %lf %[^\n]",(char *)parameter_name,
                   &parameter_value,trailing_comment)!=EOF)
     {
-
+      int nRead;
+      
       if (strcasecmp(parameter_name,"Mass")==0)
         Mass=parameter_value;
       else if (strcasecmp(parameter_name,"I_xx")==0)
@@ -461,15 +471,16 @@ DotAirLoader::DotAirLoader(const char* filename) /*{{{*/
         done_parsing_gear=false;
         while(!done_parsing_gear)
         {
-          (void)fscanf(airplane_data,"%s %[^\n]",(char *)parameter_name,
-                       trailing_comment);
-
+          nRead = fscanf(airplane_data,"%s %[^\n]",(char *)parameter_name,
+                         trailing_comment);
+          Check_fscanf(nRead, 1);
+          
           if (strcasecmp(parameter_name,"locations")==0)
           {
             for (loop=0;loop<num_wheels;loop++)
             {
-              (void)fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
-              
+              nRead = fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
+              Check_fscanf(nRead, 3);
               wheels[loop].v_P = CRRCMath::Vector3((double)p1, (double)p2, (double)p3);
               
               // track wingspan
@@ -487,7 +498,8 @@ DotAirLoader::DotAirLoader(const char* filename) /*{{{*/
           {
             for (loop=0;loop<num_wheels;loop++)
             {
-              (void)fscanf(airplane_data,"%f",&p1);
+              nRead = fscanf(airplane_data,"%f",&p1);
+              Check_fscanf(nRead, 1);
               wheels[loop].spring_constant = (SCALAR)p1;
             }
           }
@@ -495,7 +507,8 @@ DotAirLoader::DotAirLoader(const char* filename) /*{{{*/
           {
             for (loop=0;loop<num_wheels;loop++)
             {
-              (void)fscanf(airplane_data,"%f",&p1);
+              nRead = fscanf(airplane_data,"%f",&p1);
+              Check_fscanf(nRead, 1);
               wheels[loop].spring_damping=(SCALAR)p1;
             }
           }
@@ -503,7 +516,8 @@ DotAirLoader::DotAirLoader(const char* filename) /*{{{*/
           {
             for (loop=0;loop<num_wheels;loop++)
             {
-              (void)fscanf(airplane_data,"%f",&p1);
+              nRead = fscanf(airplane_data,"%f",&p1);
+              Check_fscanf(nRead, 1);
               wheels[loop].percent_brake=(SCALAR)p1;
             }
           }
@@ -511,7 +525,8 @@ DotAirLoader::DotAirLoader(const char* filename) /*{{{*/
           {
             for (loop=0;loop<num_wheels;loop++)
             {
-              (void)fscanf(airplane_data,"%f %f",&p1,&p2);
+              nRead = fscanf(airplane_data,"%f %f",&p1,&p2);
+              Check_fscanf(nRead, 2);
               wheels[loop].steering_mapping   = (int)p1;
               wheels[loop].steering_max_angle = (int)p2;
               wheels[loop].caster_angle_rad   = 0;
@@ -528,9 +543,9 @@ DotAirLoader::DotAirLoader(const char* filename) /*{{{*/
         fSectionParsingDone = false;
         while (!fSectionParsingDone)
         {
-          (void)fscanf(airplane_data,"%s %[^\n]",(char *)parameter_name,
-                       trailing_comment);
-
+          nRead = fscanf(airplane_data,"%s %[^\n]",(char *)parameter_name,
+                         trailing_comment);
+          Check_fscanf(nRead, 1);
           fSectionParsingDone = (strcasecmp(parameter_name,"end_triangles")==0);
         }
       }
@@ -539,9 +554,9 @@ DotAirLoader::DotAirLoader(const char* filename) /*{{{*/
         fSectionParsingDone = false;
         while (!fSectionParsingDone)
         {
-          (void)fscanf(airplane_data,"%s %[^\n]",(char *)parameter_name,
-                       trailing_comment);
-
+          nRead = fscanf(airplane_data,"%s %[^\n]",(char *)parameter_name,
+                         trailing_comment);
+          Check_fscanf(nRead, 1);
           fSectionParsingDone = (strcasecmp(parameter_name,"end_sphere")==0);
         }
       }
@@ -550,9 +565,9 @@ DotAirLoader::DotAirLoader(const char* filename) /*{{{*/
         fSectionParsingDone = false;
         while (!fSectionParsingDone)
         {
-          fscanf(airplane_data,"%s %[^\n]",(char *)parameter_name,
-                 trailing_comment);
-
+          nRead = fscanf(airplane_data,"%s %[^\n]",(char *)parameter_name,
+                         trailing_comment);
+          Check_fscanf(nRead, 1);
           fSectionParsingDone = (strcasecmp(parameter_name,"end_cylinder")==0);
         }
       }
@@ -561,9 +576,9 @@ DotAirLoader::DotAirLoader(const char* filename) /*{{{*/
         fSectionParsingDone = false;
         while (!fSectionParsingDone)
         {
-          fscanf(airplane_data,"%s %[^\n]",(char *)parameter_name,
-                 trailing_comment);
-
+          nRead = fscanf(airplane_data,"%s %[^\n]",(char *)parameter_name,
+                         trailing_comment);
+          Check_fscanf(nRead, 1);
           fSectionParsingDone = (strcasecmp(parameter_name,"end_extrusion")==0);
         }
       }
