@@ -74,6 +74,8 @@ HardPointRotation::HardPointRotation(SimpleXMLTransfer *xml, TSimInputs const& i
       // The "*2" factor for each gain value scales the control input
       // values from -0.5...+0.5 to -1.0...+1.0. This saves one
       // float multiplication per mapping in the runtime update() routine.
+      // NB: unsigned control (throttle, spoiler, retract) are not scaled,
+      //     since control input is already in range 0.0...+1.0
       std::string mapping = child->getString("mapping", "NOTHING");
       float gain = (float)child->getDouble("gain", 1.0);
       std::cout << "  mapped to " << mapping << " with gain " << gain;
@@ -93,7 +95,7 @@ HardPointRotation::HardPointRotation(SimpleXMLTransfer *xml, TSimInputs const& i
       else if (mapping == "THROTTLE")
       {
         datasource.push_back(&in.throttle);
-        source_gain.push_back(gain * 2);
+        source_gain.push_back(gain);
         num_controls++;
       }
       else if (mapping == "RUDDER")
@@ -111,13 +113,13 @@ HardPointRotation::HardPointRotation(SimpleXMLTransfer *xml, TSimInputs const& i
       else if (mapping == "SPOILER")
       {
         datasource.push_back(&in.spoiler);
-        source_gain.push_back(gain * 2);
+        source_gain.push_back(gain);
         num_controls++;
       }
       else if (mapping == "RETRACT")
       {
         datasource.push_back(&in.retract);
-        source_gain.push_back(gain * 2);
+        source_gain.push_back(gain);
         num_controls++;
       }
       else if (mapping == "PITCH")
@@ -521,7 +523,7 @@ void WheelSystem::update( TSimInputs* inputs,
  * Create a WheelSystem with no hardpoints
  */
 WheelSystem::WheelSystem()
-: v_Forces(0, 0, 0), v_Moments(0,0,0), dMaxSize(0), span_ft(0), dZLow(0) 
+: v_Forces(0, 0, 0), v_Moments(0,0,0), span_ft(0), dZLow(0), dZHigh(0) 
 {
 }
 
@@ -537,7 +539,6 @@ void WheelSystem::init(SimpleXMLTransfer *ModelFile, SCALAR  def_span)
   SimpleXMLTransfer  *e, *i;
   unsigned int       uSize;
   double             x, y, z;
-  double             dist;
   double             to_ft;
   double             to_lbf_per_ft;
   double             to_lbf_s_per_ft;
@@ -568,9 +569,6 @@ void WheelSystem::init(SimpleXMLTransfer *ModelFile, SCALAR  def_span)
   dZLow    = 0;
   dZHigh   = 0;
   
-  // let's assume that there is nothing distant from the CG:
-  dMaxSize = 0;
-  
   wheels.clear();
   
   i = ModelFile->getChild("wheels");
@@ -662,12 +660,7 @@ void WheelSystem::init(SimpleXMLTransfer *ModelFile, SCALAR  def_span)
     // highest point?
     if (dZHigh > z)
       dZHigh = z;
-    // far away (Z distance is assumed to be low)?
-    dist = x*x + y*y;
-    if (dist > dMaxSize)
-      dMaxSize = dist;
   }
-  dMaxSize = sqrt(dMaxSize);
   span_ft  = 2 * span;
 
   // just in case: if there were no hardpoints, use the reference span
