@@ -31,9 +31,10 @@ void Power::SimpleThrust::ReloadParams(SimpleXMLTransfer* xml)
 {
   Gearing::ReloadParams(xml);
   
-  std::cout <<"      SimpleThrust\n";
   k_F = xml->getDouble("k_F");
   k_M = xml->getDouble("k_M");
+  
+  std::cout << "SimpleThrust: k_F=" << k_F << ", k_M=" << k_M << "\n";
 }
 
 void Power::SimpleThrust::step(PowerValuesStep* values)
