@@ -29,7 +29,7 @@
 
 
 #define MOD_FDM_USE_LARCSIM     1
-#define MOD_FDM_USE_002         1
+#define MOD_FDM_USE_002         0   // should not be used anymore, since aero model is out-of-date
 #define MOD_FDM_USE_DISPLAYMODE 1
 #define MOD_FDM_USE_TESTMODE    1
 #define MOD_FDM_USE_HELI01      1
