@@ -338,8 +338,9 @@ void T_TX_Interface::update(double dt)
  *  Mixers are enabled but not effective.
  */
 T_TX_Mixer::T_TX_Mixer(T_TX_Interface *parent) 
+  : child_in_cfg(""), iface(parent)
 {
-  baseInit(parent);
+  baseInit();
 #if DEBUG_TX_INTERFACE > 0
   printf("T_TX_Mixer::T_TX_Mixer()\n");
 #endif
@@ -358,8 +359,9 @@ T_TX_Mixer::T_TX_Mixer(T_TX_Interface *parent)
  */
 T_TX_Mixer::T_TX_Mixer(T_TX_Interface *parent, 
                        SimpleXMLTransfer* cfg, std::string child)
+  : child_in_cfg(""), iface(parent)
 {
-  baseInit(parent);
+  baseInit();
 #if DEBUG_TX_INTERFACE > 0
   printf("T_TX_Mixer::T_TX_Mixer(cfg, child)\n");
 #endif
@@ -381,7 +383,7 @@ T_TX_Mixer::~T_TX_Mixer()
 
 /** \brief Initialize all variables
  */
-void T_TX_Mixer::baseInit(T_TX_Interface *iface_val)
+void T_TX_Mixer::baseInit()
 {
   enabled = 1;
   dr_enabled = 0; // dual-rate off
@@ -403,8 +405,6 @@ void T_TX_Mixer::baseInit(T_TX_Interface *iface_val)
     mixer_src[i] = mixer_dst[i] = T_AxisMapper::NOTHING;
     mixer_val[i] = 0.0;
   }
-
-  iface = iface_val;
 }
 
 
@@ -414,32 +414,53 @@ void T_TX_Mixer::baseInit(T_TX_Interface *iface_val)
  *  This file may contain more than one branch with interface settings,
  *  so the name of the child has to be specified.
  */
-int T_TX_Mixer::init(SimpleXMLTransfer* cfg, std::string child)
+int T_TX_Mixer::init(SimpleXMLTransfer* cfg, std::string childname)
 {
 #if DEBUG_TX_INTERFACE > 0
   printf("T_TX_Mixer::init(cfg, child)\n");
-  printf(" <-- %s\n", child.c_str());
+  printf(" <-- %s\n", childname.c_str());
 #endif
 
-  SimpleXMLTransfer* mixer;
-  SimpleXMLTransfer* item;
-  SimpleXMLTransfer* inter;
-
   // store the child's name for writing back the settings
-  child_in_cfg = child;
+  child_in_cfg = childname;
+
+  return reinit(cfg, false);
+}
+
+
+/** \brief Re-initialize the mixer from a config file.
+ *
+ *  The mixer object will be re-initialized from the given config file.
+ *  If fIsChild = true the file only contains the mixer child.
+ *  The interface, hence the name of the interface child, are unchanged.
+ */
+int T_TX_Mixer::reinit(SimpleXMLTransfer* cfg, bool fIsChild)
+{
+  SimpleXMLTransfer* mixer;
+
+  if (fIsChild)
+  {
+#if DEBUG_TX_INTERFACE > 0
+    printf("T_TX_Mixer::reinit(cfg, fIsChild)\n");
+#endif
+    mixer = cfg;
+  }
+  else
+  {
+    SimpleXMLTransfer* inter = cfg->getChild(child_in_cfg, true);
+    mixer = inter->getChild("mixer", true);
+  }
+
+  printf("Loading mixer settings from %s:\n", child_in_cfg.c_str());
 
-  printf("Loading mixer settings from %s:\n", child.c_str());
   try
   {
-    inter = cfg->getChild(child, true);
-    mixer = inter->getChild("mixer", true);
-
     enabled = mixer->attributeAsInt("enabled", 1);
     dr_enabled = mixer->attributeAsInt("dr_enabled", 0);
  
     for (int i = T_AxisMapper::AILERON; i <= T_AxisMapper::PITCH; i++)
     { 
-      item = mixer->getChild(Global::inputDev->AxisStringsXML[i], true);
+      SimpleXMLTransfer* item = mixer->getChild(Global::inputDev->AxisStringsXML[i], true);
 
       trim_val[i]   = item->getDouble("trim",   0.0);
       nrate_val[i]  = item->getDouble("nrate",  1.0);
@@ -452,7 +473,8 @@ int T_TX_Mixer::init(SimpleXMLTransfer* cfg, std::string child)
       
     for (int i = 0; i < T_TX_Mixer::NUM_MIXERS; i++)
     { 
-      item = mixer->getChild(Global::inputDev->MixerStringsXML[i], true);
+      SimpleXMLTransfer* item = mixer->getChild(Global::inputDev->MixerStringsXML[i], true);
+      
       mixer_enabled[i] = item->attributeAsInt("enabled", 0);
       mixer_src[i]     = item->getDouble("src", T_AxisMapper::NOTHING);
       mixer_dst[i]     = item->getDouble("dst", T_AxisMapper::NOTHING);
@@ -873,7 +895,7 @@ T_Calibration::T_Calibration(T_TX_Interface *parent)
 T_Calibration::T_Calibration(T_TX_Interface *parent,
                              SimpleXMLTransfer* cfg,
                              std::string child)
-  : child_in_cfg(child), iface(parent)
+  : child_in_cfg(""), iface(parent)
 {
 #if DEBUG_TX_INTERFACE > 0
   printf("T_Calibration::T_Calibration(cfg, child)\n");
