@@ -36,6 +36,7 @@
 #include <crrc_config.h>
 
 #include "crrc_builtin_scenery.h"
+#include "../global.h"
 #include "../crrc_main.h"
 #include "../ImageLoaderTGA.h"
 #include "../mod_misc/filesystools.h"
@@ -409,41 +410,40 @@ void BuiltinScenery::compile_display_list()
   glEndList();
 }
 
-/// \todo these defines are currently defined here and in crrc_graphics.cpp.
-///       In the end these values should come from the scenery file, and
-///       the Scenery base class must offer an interface to get it.
-#define LIGHT_X (1000.0f / 1.72)
-#define LIGHT_Y (8000.0f / 1.72)
-#define LIGHT_Z (-1000.0f / 1.72)
 void BuiltinScenery::setup_drawing_state()
 {
   GLfloat mat_specular[]={1.0,1.0,1.0,1.0};
   GLfloat mat_diffuse[]={0.3,0.3,0.3,1.0};
-  GLfloat light_position[]={0.0,LIGHT_X,LIGHT_Y,LIGHT_Z};
-  GLfloat white_light[]={1.0,1.0,1.0,1.0};
+  
+  sgVec4 ambient_light;
+  Global::scenery->getAmbientColor(ambient_light);
+  sgVec4 sun_light;
+  Global::scenery->getSunColor(sun_light);
+  sgVec3 sun_pos;
+  Global::scenery->getSunPosition(sun_pos);
 
   glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
   glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,mat_specular);
   glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,mat_diffuse);
-  glLightfv(GL_LIGHT0,GL_POSITION,light_position);
-  glLightfv(GL_LIGHT0,GL_DIFFUSE,white_light);
-  glLightfv(GL_LIGHT0,GL_SPECULAR,white_light);
+  glLightfv(GL_LIGHT0,GL_AMBIENT,ambient_light);
+  glLightfv(GL_LIGHT0,GL_DIFFUSE,sun_light);
+  glLightfv(GL_LIGHT0,GL_SPECULAR,sun_light);
+  glLightfv(GL_LIGHT0,GL_POSITION,sun_pos);
   glBlendFunc(GL_ONE,GL_ONE_MINUS_SRC_COLOR);
   glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_BLEND);
-  glTexEnvfv(GL_TEXTURE_ENV,GL_TEXTURE_ENV_COLOR,white_light);
+  glTexEnvfv(GL_TEXTURE_ENV,GL_TEXTURE_ENV_COLOR,sun_light);
   glDisable(GL_COLOR_MATERIAL);
 
-  glEnable(GL_LIGHTING);
-  glEnable(GL_LIGHT0);
-  glEnable(GL_TEXTURE_2D);
-  glEnable(GL_NORMALIZE);
-  glDisable(GL_CULL_FACE);
-
   // viewing transformation
   gluLookAt(player_pos.r[0], player_pos.r[1], player_pos.r[2],
             Video::looking_pos.r[0], Video::looking_pos.r[1], Video::looking_pos.r[2],
             0.0, 1.0, 0.0);
 
+  glEnable(GL_LIGHTING);
+  glEnable(GL_LIGHT0);
+  glEnable(GL_TEXTURE_2D);
+  glEnable(GL_NORMALIZE);
+  glDisable(GL_CULL_FACE);
 }
 
 void BuiltinScenery::restore_drawing_state()
@@ -726,16 +726,22 @@ float BuiltinSceneryDavis::getHeightAndPlane(float x_north, float y_east, float
   tplane[2] = 0.0;
   tplane[3] = 0.1;//0.0;
 
-  return getHeight( x_north,  y_east);
+  return getHeight(x_north, y_east);
 }
-/***/
-void BuiltinSceneryDavis::getWindComponents(double X_cg,double  Y_cg,double  Z_cg,
-    float  *x_wind_velocity, float  *y_wind_velocity, float  *z_wind_velocity)
+
+
+/** \brief Get wind components at position X_cg, Y_cg, Z_cg
+ *  
+ */
+int BuiltinSceneryDavis::getWindComponents(double X_cg, double Y_cg, double Z_cg,
+    float *x_wind_velocity, float *y_wind_velocity, float *z_wind_velocity)
 {
-  float    flWindVel = cfg->wind->getVelocity();
+  float flWindVel = cfg->wind->getVelocity();
   *x_wind_velocity = -1 * flWindVel * cos(M_PI*cfg->wind->getDirection()/180);
   *y_wind_velocity = -1 * flWindVel * sin(M_PI*cfg->wind->getDirection()/180);
   *z_wind_velocity = 0.;
+  
+  return 0;
 }
 
 
@@ -1608,12 +1614,14 @@ BuiltinSceneryCapeCod::BuiltinSceneryCapeCod(SimpleXMLTransfer *xml, int sky_var
     free(sand_texture);
   }
 }
-/***************************/
-void BuiltinSceneryCapeCod::getWindComponents(double X_cg,double  Y_cg,double  Z_cg,
-    float  *x_wind_velocity, float  *y_wind_velocity, float  *z_wind_velocity)
-{
 
-  float    flWindVel = cfg->wind->getVelocity();
+/** \brief Get wind components at position X_cg, Y_cg, Z_cg
+ *  
+ */
+int BuiltinSceneryCapeCod::getWindComponents(double X_cg, double Y_cg, double Z_cg,
+    float *x_wind_velocity, float *y_wind_velocity, float *z_wind_velocity)
+{
+  float flWindVel = cfg->wind->getVelocity();
   *x_wind_velocity = -1 * flWindVel * cos(M_PI*cfg->wind->getDirection()/180);
   *y_wind_velocity = -1 * flWindVel * sin(M_PI*cfg->wind->getDirection()/180);
   *z_wind_velocity = 0.;
@@ -1655,6 +1663,8 @@ void BuiltinSceneryCapeCod::getWindComponents(double X_cg,double  Y_cg,double  Z
       }
     }
   }
+  
+  return 0;
 }
 
 
