@@ -28,9 +28,7 @@
 #define CRRC_WINDFROMTERRAIN_H
 
  
-#include <plib/ssg.h>
- 
-void wind_from_terrain(double X, double Y, double Z,
+int wind_from_terrain(double X, double Y, double Z,
     float *x_wind_velocity, float *y_wind_velocity, float *z_wind_velocity);
     
 #endif //CRRC_WINDFROMTERRAIN_H
