@@ -31,7 +31,7 @@
  *          crrcsim 0.8.1 and for windows only. The last release of this specific
  *          branch is the version 1.5.1.
  */
-
+#include "../../i18n.h"
 #include "../../include_gl.h"
 
 #include <stdlib.h>
@@ -47,9 +47,11 @@
 #include "../../mod_misc/filesystools.h"
 #include "../../mod_landscape/crrc_scenery.h"
 #include "handlerF3F.h"
+#include "../../SimStateHandler.h"
 #include "../../record.h"
 #include "../../robots.h"
 #include "../../mod_robots/marker.h"
+#include "../../mod_video/fonts.h"
 #include <iostream>
 using namespace std;
 
@@ -108,6 +110,13 @@ HandlerF3F::HandlerF3F()
   pylon_rendering_state->enable(GL_LIGHTING);
   pylon_rendering_state->disable(GL_BLEND);
 
+  plane_rendering_state = new ssgSimpleState();
+  plane_rendering_state->disable(GL_CULL_FACE);
+  plane_rendering_state->disable(GL_COLOR_MATERIAL);
+  plane_rendering_state->disable(GL_TEXTURE_2D);
+  plane_rendering_state->disable(GL_LIGHTING);
+  plane_rendering_state->enable(GL_BLEND);
+
   text_rendering_state = new ssgSimpleState();
   text_rendering_state->disable(GL_CULL_FACE);
   text_rendering_state->disable(GL_COLOR_MATERIAL);
@@ -145,27 +154,48 @@ HandlerF3F::~HandlerF3F()
  */
 void HandlerF3F::draw()
 {
-  pylon_rendering_state->apply();
+  if (Global::testmode)
+    return;
+    
   GLfloat x1, x2, y1, y2;
   y1 =  0;
   y2 = -10;
   x1 = plan_limit;
   x2 = -plan_limit;
 
-  draw_f3f_base_xy (x1, y1);
-  draw_f3f_base_xy (x1, y2);
-  draw_f3f_base_xy (x2, y1);
-  draw_f3f_base_xy (x2, y2);
-
+  pylon_rendering_state->apply();
+  draw_f3f_base_xy(x1, y1);
+  draw_f3f_base_xy(x1, y2);
+  draw_f3f_base_xy(x2, y1);
+  draw_f3f_base_xy(x2, y2);
+  if (extend_base == TRUE) 
+  {
+    plane_rendering_state->apply();
+    draw_f3f_base_plane_xy(x1, YY_cg_rwy, 0);
+    draw_f3f_base_plane_xy(x2, YY_cg_rwy, 1);
+  }
 }
+
 void HandlerF3F::draw_f3f_base_xy(GLfloat x, GLfloat y) const
 { 
   GLfloat nord, east, z;
   nord = center_base_position_north + y * cos_dir - x * sin_dir;
-  east = center_base_position_east + y * (sin_dir) + x * cos_dir;
+  east = center_base_position_east + y * sin_dir + x * cos_dir;
   z = Global::scenery->getHeight(nord, east);
   if(z==-9999)z = -50;//???????????????
-  draw_f3f_base (nord, east, z);
+  draw_f3f_base(nord, east, z);
+}
+
+void HandlerF3F::draw_f3f_base_plane_xy(GLfloat x, GLfloat y, int base_left) const
+{ 
+  GLfloat nord, east, dn, de, z;
+  nord = center_base_position_north + y * cos_dir - x * sin_dir;
+  east = center_base_position_east + y * sin_dir + x * cos_dir;
+  z = Global::scenery->getHeight(nord, east);
+  if(z==-9999)z = -50;//???????????????
+  dn = cos_dir;
+  de = sin_dir;
+  draw_f3f_base_plane(nord, east, z, dn, de, base_left);
 }
 
 /** \brief Print the game-mode-specific text overlay
@@ -195,13 +225,23 @@ void HandlerF3F::display_infos(GLfloat ww, GLfloat hh)
   
   if (ww <= 800)
   {
-    fontRenderer.setFont(fntGetBitmapFont(FNT_BITMAP_HELVETICA_18));
+    if (Video::textureFont)
+    {
+      fontRenderer.setFont(Video::textureFont);
+      fontRenderer.setPointSize(16);
+    }
+    else fontRenderer.setFont(fntGetBitmapFont(FNT_BITMAP_HELVETICA_18));
     y_offset = 45;
     lineheight = 27;
   }
   else
   {
-    fontRenderer.setFont(fntGetBitmapFont(FNT_BITMAP_TIMES_ROMAN_24));
+    if (Video::textureFont)
+    {
+      fontRenderer.setFont(Video::textureFont);
+      fontRenderer.setPointSize(22);
+    }
+    else fontRenderer.setFont(fntGetBitmapFont(FNT_BITMAP_TIMES_ROMAN_24));
     y_offset = 55;
     lineheight = 36;
   }
@@ -237,17 +277,17 @@ output(100., 100., buf);
             f3f_score);
 
     // table header for lap statistics
-    sprintf (astring,"- Lap  RunTime  TurnTime  LostMeters -");
+    sprintf (astring,_("- Lap   RunTime   TurnTime   LostMeters -") );
     GLfloat header_length = textLength(astring);
     GLfloat a_width = textLength("a");            /// \todo rather use height("X") than width("a")
 
     // start time
-    output (window_xsize - header_length,
+    output (window_xsize/2 - header_length/2,
             window_ysize - y_offset - 8 * a_width,
-            (std::string("Run started: ") + start_time).c_str());
+            (std::string(_("Run started: ") ) + start_time).c_str());
     
     // lap statistics
-    output (window_xsize - header_length, 
+    output (window_xsize/2 - header_length/2, 
             window_ysize - y_offset - 11 * a_width,
             astring);
     for (i = 1; i <= base_count; i++) 
@@ -256,15 +296,15 @@ output(100., 100., buf);
       ilost = lostm[i];
       iturn = turntime[i];
       UPDATE_RESULT;
-      output (window_xsize - header_length,
+      output (window_xsize/2 - header_length/2,
               window_ysize - (y_offset + a_width * (11 + i * 2)), 
               f3f_result);
     }
     
     // end time
-    output (window_xsize - header_length,
+    output (window_xsize/2 - header_length/2,
             window_ysize - (y_offset + 34 * a_width), 
-            (std::string("Run finished: ") + end_time).c_str());
+            (std::string(_("Run finished: ")) + end_time).c_str());
     
   }
   else
@@ -335,7 +375,7 @@ void HandlerF3F::finishTextRendering() const
 void HandlerF3F::output(GLfloat x, GLfloat y, const char *text)
 {
   fontRenderer.begin();
-  glColor4f (1.0, 1.0, 1.0, 1.0);
+  glColor4f (1, 0.3, 0.3, 0.7);
   fontRenderer.start2f(x, y);
   fontRenderer.puts(text);
   fontRenderer.end();
@@ -364,7 +404,6 @@ GLfloat HandlerF3F::textLength(const char *text) const
 }
 
 
-
 /** \brief Draw a base pylon
  *
  *  This method draws one pole of a base pylon.
@@ -378,7 +417,7 @@ GLfloat HandlerF3F::textLength(const char *text) const
  *  \param  fy  vertical location of the pole
  *  \param  fh  height of the pole
  */
-void HandlerF3F::draw_f3f_base(GLfloat nord, GLfloat east, GLfloat  fh) const
+void HandlerF3F::draw_f3f_base(GLfloat nord, GLfloat east, GLfloat fh) const
 { 
   int use_textures=1;
   static GLuint f3f_baseTexture;
@@ -477,6 +516,7 @@ void HandlerF3F::draw_f3f_base(GLfloat nord, GLfloat east, GLfloat  fh) const
 
     glMaterialfv(GL_FRONT,GL_AMBIENT,red_base);
     glMaterialfv(GL_FRONT,GL_DIFFUSE,red_base);
+/*
     if (extend_base == TRUE) 
     {
       glPushMatrix();
@@ -485,10 +525,90 @@ void HandlerF3F::draw_f3f_base(GLfloat nord, GLfloat east, GLfloat  fh) const
       Video::drawSolidCube(3);
       glPopMatrix();
     }
+*/
   }
 }
 
 
+/** \brief Draw a base plane
+ *
+ *  This method draws one large semi transparent plane on the base.
+ *
+ *  \param  fx  horizontal location of the pole
+ *  \param  fy  vertical location of the pole
+ *  \param  fh  height of the pole
+ */
+void HandlerF3F::draw_f3f_base_plane(GLfloat nord, GLfloat east, GLfloat fh, GLfloat dn, GLfloat de, int base_left) const
+{
+  GLfloat size = 10;
+  GLfloat fx = east;
+  GLfloat fy = -nord;
+  GLfloat dx = size*de;
+  GLfloat dy = -size*dn;
+  GLfloat black_base_alpha[] = {0,0,0, 0.2};
+  GLfloat red_base_alpha[]   = {1,0,0, 0.2};
+  GLfloat white_base_alpha[] = {1,1,1, 0.2};
+
+  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+
+  glPushMatrix();
+  glTranslatef(fx,ZZ_cg_rwy,fy);
+
+  glColor4fv(black_base_alpha);
+
+  if (run_started && !run_completed)
+  {
+    if ((next_base_to_cross == PYLON_RIGHT) && !base_left)
+      glColor4fv(red_base_alpha);
+    if ((next_base_to_cross == PYLON_LEFT) && base_left)
+      glColor4fv(red_base_alpha);
+  }
+  else if (run_will_start && !run_started)
+  {
+    if ((next_base_to_cross == PYLON_RIGHT) && base_left)
+      glColor4fv(red_base_alpha);
+    if ((next_base_to_cross == PYLON_LEFT) && !base_left)
+      glColor4fv(red_base_alpha);
+  }
+  else if (!run_will_start)
+  {
+    if ((start_on_left == TRUE) && base_left)
+      glColor4fv(red_base_alpha);
+    if ((start_on_left == FALSE) && !base_left)
+      glColor4fv(red_base_alpha);
+  }
+    
+  glBegin(GL_QUADS);
+  glVertex3f(  0,    0,0);
+  glVertex3f(  0,-size,0);
+  glVertex3f(-dx,-size,-dy);
+  glVertex3f(-dx,    0,-dy);
+  glEnd();
+  glBegin(GL_QUADS);
+  glVertex3f(  0,    0,0);
+  glVertex3f(  0, size,0);
+  glVertex3f( dx, size,dy);
+  glVertex3f( dx,    0,dy);
+  glEnd();
+
+  glColor4fv(white_base_alpha);
+  glBegin(GL_QUADS);
+  glVertex3f(  0,    0,0);
+  glVertex3f(  0, size,0);
+  glVertex3f(-dx, size,-dy);
+  glVertex3f(-dx,    0,-dy);
+  glEnd();
+  glBegin(GL_QUADS);
+  glVertex3f(  0,    0,0);
+  glVertex3f(  0,-size,0);
+  glVertex3f( dx,-size,dy);
+  glVertex3f( dx,    0,dy);
+  glEnd();
+
+  glPopMatrix();
+}
+
+
 /** \brief Reset the game handler.
  *
  *  Everything will be reset to be ready for a new run.
@@ -603,10 +723,8 @@ int HandlerF3F::play (std::string soundName, int base)
  */
 void HandlerF3F::update_time ()
 {
-  // *** to replace by the parameter if needed
-  // *** preferable use the exact clock at the time of execution
-  currtime = SDL_GetTicks() - resettime;
-  runtime = currtime - runstarttime - pausetime;
+  currtime = Global::Simulation->getSimulationTimeSinceReset();
+  runtime = currtime - runstarttime;
 }
 
 
@@ -787,6 +905,9 @@ void HandlerF3F::update_turntime ()
  */
 void HandlerF3F::update(float a, float b, float c, FlightRecorder* recorder, Robots* robots)
 {
+  if (Global::testmode)
+    return;
+    
   //convert coords on F3F coords
   a -= center_base_position_north;
   b -= center_base_position_east;
