@@ -44,39 +44,55 @@
 #include "../crrc_system.h"
 #include "../defines.h"
 #include "../mod_landscape/crrc_scenery.h"
-#include "crrc_sky.h"
 #include "glconsole.h"
 #include "gloverlay.h"
 #include "../zoom.h"
 #include "fonts.h"
+#include "../mod_misc/filesystools.h"
+
+// model close-view tuning parameters
+#define MODEL_VIEW_SIZE        0.15 // fraction of screen used up by close-view window
+#define MODEL_VIEW_FOV_MARGIN  0.3  // extra FOV width (relative)
+
+// smart camera tuning parameters
+#define SMARTCAM_MAX_FRACTION  0.6 // maximum fraction of screen used up by smart camera
+#define SMARTCAM_TIME          60. // smart camera adaptation time (s)
+#define SMARTCAM_RANGE_FAC     1.3 // smart camera range = 4/3 sigma
 
 // Debug and error handling settings
 #define DONT_REPEAT_GL_ERRORS  1
 
+
 namespace Video
 {
+bool ssgLoadJPG(const char *fname, ssgTextureInfo* info);
+
+fntTexFont   *textureFont; //textured font 
+std::string codeSet; //for font filename construct
 
 // Flags for SDL video mode
 Uint32 SDL_video_flags;
 
-#define LIGHT_X (1000.0f / 1.72)
-#define LIGHT_Y (8000.0f / 1.72)
-#define LIGHT_Z (-1000.0f / 1.72)
-ssgRoot               *scene    = NULL ;
-static ssgContext     *context  = NULL ;
-sgVec3                lightposn = {LIGHT_X, LIGHT_Y, LIGHT_Z};
+static ssgContext *context = NULL ;
 
 int window_xsize;  // Size of window in x direction
 int window_ysize;  // Size of window in y direction
 int screen_xsize;  // Size of screen in x direction
 int screen_ysize;  // Size of screen in y direction
 
-float shadow_matrix[4][4];  // Matrix used to calculate shadow on ground
-float slope_shadow_matrix[4][4];  // Matrix used to calculate shadow on slope
-float top_shadow_matrix[4][4];  // Matrix used to calculate shadow on slope
-
 T_VideoBitDepthInfo vidbits;
 
+/**
+ * Smart camera setting
+ * and smart camera center/range
+ */
+float flSmartCam = SMARTCAM_MAX_FRACTION;
+float flSmartCam_time;
+float flSmartCam_center_w;
+float flSmartCam_center_h;
+float flSmartCam_var_w;
+float flSmartCam_var_h;
+
 /**
  * Sloppy camera setting
  */
@@ -84,7 +100,7 @@ float flSloppyCam = 0;
 
 /**
  * Position where the player is looking at (may be different
- * from position of plane; needed for sloppy camera).
+ * from position of plane; needed for sloppy & smart camera).
  */
 CRRCMath::Vector3 looking_pos;
 
@@ -156,56 +172,13 @@ bool evaluateOpenGLErrors()
 }
 
 
-/*****************************************************************************/
-/** \brief Calculate the shadow matrix
- *
- *  The shadow matrix projects the shadow-casting object
- *  onto the "ground" plane (given as Ax + By + Cz + D = 0).
- *
- *  This technique is described in 
- *  http://www.opengl.org/resources/tutorials/advanced/advanced98/notes/notes.html
- *
- *  \param ground (A, B, C, D) of ground plane
- *  \param light  light source position in homogenous coordinates (with w = 0 for a directional light)
- *  \param m Shadow matrix will be copied to this array
- */
-static void ShadowMatrix(float ground[4],float light[4],float m[4][4])
-{
-  float dot;
-
-  dot = ground[0] * light[0] +
-        ground[1] * light[1] +
-        ground[2] * light[2] +
-        ground[3] * light[3];
-
-  m[0][0] = dot - light[0] * ground[0];
-  m[1][0] = 0.0 - light[0] * ground[1];
-  m[2][0] = 0.0 - light[0] * ground[2];
-  m[3][0] = 0.0 - light[0] * ground[3];
-
-  m[0][1] = 0.0 - light[1] * ground[0];
-  m[1][1] = dot - light[1] * ground[1];
-  m[2][1] = 0.0 - light[1] * ground[2];
-  m[3][1] = 0.0 - light[1] * ground[3];
-
-  m[0][2] = 0.0 - light[2] * ground[0];
-  m[1][2] = 0.0 - light[2] * ground[1];
-  m[2][2] = dot - light[2] * ground[2];
-  m[3][2] = 0.0 - light[2] * ground[3];
-
-  m[0][3] = 0.0 - light[3] * ground[0];
-  m[1][3] = 0.0 - light[3] * ground[1];
-  m[2][3] = 0.0 - light[3] * ground[2];
-  m[3][3] = dot - light[3] * ground[3];
-}
-
-
 /*****************************************************************************/
 unsigned short getshort(FILE *inf)
 {
   unsigned char buf[2];
   
-  (void)fread(buf,2,1,inf);
+  int nres = fread(buf,2,1,inf);
+  if (nres != 1) return 0;
   return (buf[0]<<8)+(buf[1]<<0);
 }
 
@@ -214,7 +187,8 @@ unsigned short getrgbchar(FILE *inf)
 {
   unsigned char buf[1];
 
-  (void)fread(buf,1,1,inf);
+  int nres = fread(buf,1,1,inf);
+  if (nres != 1) return 0;
   return (buf[0]);
 }
 
@@ -273,23 +247,28 @@ unsigned char * read_rgbimage(const char *name, int *w, int *h)
         input_short=getshort(image_in);
         if (input_short == 4)
         {
-          (void)fread(header,sizeof(unsigned char),500,image_in);
-          (void)fread(temp, sizeof image[0], *w * *h, image_in);
+          int nres = fread(header,sizeof(unsigned char),500,image_in);
+          if (nres != 500) return NULL;
+          nres = fread(temp, sizeof image[0], *w * *h, image_in);
+          if (nres != (*w * *h)) return NULL;
           for (loop=0;loop<(unsigned long int)(*w * *h);loop++)
           {
             image[loop*4+0]=temp[loop];
           }
-          (void)fread(temp, sizeof image[0], *w * *h, image_in);
+          nres = fread(temp, sizeof image[0], *w * *h, image_in);
+          if (nres != (*w * *h)) return NULL;
           for (loop=0;loop<(unsigned long int)(*w * *h);loop++)
           {
             image[loop*4+1]=temp[loop];
           }
-          (void)fread(temp, sizeof image[0], *w * *h, image_in);
+          nres = fread(temp, sizeof image[0], *w * *h, image_in);
+          if (nres != (*w * *h)) return NULL;
           for (loop=0;loop<(unsigned long int)(*w * *h);loop++)
           {
             image[loop*4+2]=temp[loop];
           }
-          (void)fread(temp, sizeof image[0], *w * *h, image_in);
+          nres = fread(temp, sizeof image[0], *w * *h, image_in);
+          if (nres != (*w * *h)) return NULL;
           for (loop=0;loop<(unsigned long int)(*w * *h);loop++)
           {
             image[loop*4+3]=temp[loop];
@@ -418,8 +397,6 @@ GLuint make_texture(unsigned char *pixel_data, GLint pixel_format, GLint format,
 }
 
 
-bool ssgLoadJPG ( const char *fname, ssgTextureInfo* info );
-
 /** \brief Perform the basic scenegraph initialization
  *
  *  Initializes the PLIB SSG library, creates the scenegraph
@@ -431,17 +408,39 @@ void initialize_scenegraph()
   ssgInit();
   
   // add to SSG function for read JPEG Textures 
-  ::ssgAddTextureFormat ( ".jpg", ssgLoadJPG);
+  ::ssgAddTextureFormat(".jpg", ssgLoadJPG);
   
   // font
-  puSetDefaultFonts ( FONT_HELVETICA_14, FONT_HELVETICA_14 );
+  {
+    #define FONT_FILE_PREFIX  "textures/Sans_"
+    #define FONT_FILE_SUFFIX  ".txf"
+    std::string name = FONT_FILE_PREFIX+codeSet+FONT_FILE_SUFFIX;
+    textureFont = new fntTexFont ();
+    std::string fname = FileSysTools::getDataPath(name.c_str());
+    if ((fname == "") || 
+        (textureFont->load( fname.c_str(),  GL_LINEAR, GL_LINEAR ) == FNT_FALSE))
+    {
+      std::cout << "Unable to find font " << name << "("<<fname<< "), falling back to bitmap font!" << std::endl;
+      puSetDefaultFonts ( FONT_HELVETICA_14, FONT_HELVETICA_14 );
+      textureFont = NULL;
+    }
+    else
+    {
+      std::cout << "Textured font :" << fname<< std::endl;
+      textureFont->setGap(0);
+      puFont *myfont;
+      myfont  = new puFont ( textureFont, 15 ) ;
+      puSetDefaultFonts ( *myfont, *myfont );
+    }
+  }
   
   // Some basic OpenGL setup
-  sgVec4 skycol;
-  sgSetVec4 ( skycol, 0.0f, 0.0f, 0.0f, 1.0f ) ;
-  glClearColor ( skycol[0], skycol[1], skycol[2], skycol[3] ) ;
+  glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
+  glEnable(GL_DEPTH_TEST);
 
-  glEnable ( GL_DEPTH_TEST ) ;
+  // TWO_SIDE light model is required to correctly display
+  // thin two-sided surfaces (e.g. in airplane models)
+  glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 1.0);
 
   // Set up the viewing parameters
   context = new ssgContext();
@@ -451,105 +450,53 @@ void initialize_scenegraph()
 
   ssgModelPath("");
   ssgTexturePath("textures");  
-  
-  // Create a root node
-  scene    = new ssgRoot();
-
-  // Set up a light source
-  sgVec4 lightamb;
-  sgSetVec4(lightamb , 0.2f, 0.2f, 0.2f, 1.0f);
-  ssgGetLight(0)->setPosition(lightposn);
-  ssgGetLight(0)->setColour(GL_AMBIENT, lightamb);
-
-  ssgGetLight(0)->setPosition(lightposn);
-
-
-}
-
-
-/** \brief Setup some basic OpenGL stuff
- *
- *  Calculates the shadow matrices for the different locations,
- *  performs some basic OpenGL initialization and creates the
- *  sky sphere object. Most of this stuff will get obsolete
- *  very soon...
- *
- *  \param boFlatShading  use flat shading instead of smooth shading
- */
-void initialize_window(bool boFlatShading)
-{
-  GLfloat mat_specular[]={1.0,1.0,1.0,1.0};
-  GLfloat mat_diffuse[]={0.3,0.3,0.3,1.0};
-  GLfloat light_position[]={0.0,5000.0,3000.0,0.0};
-  GLfloat white_light[]={1.0,1.0,1.0,1.0};
-  GLfloat ground[4];
-  GLfloat light[4];
-
-  ground[0]=0;
-  ground[1]=1;
-  ground[2]=0;
-  ground[3]=0;
-
-  /// \todo The light initialization seems to depend
-  /// on the scenery, but it is only done once at
-  /// startup and not after a scenery change.
-  if (Global::scenery->getID() == Scenery::CAPE_COD)
-  {
-    light[0] = 0;
-    light[1] = 5000;
-    light[2] = 0;
-    light[3] = 0.0;
-  }
-  else
-  {
-    light[0] = 0;
-    light[1] = 5000.;
-    light[2] = -3000.;
-    light[3] = 0.0;
-  }
-  ShadowMatrix(ground,light,shadow_matrix);
-
-  ground[0] = -1;
-  ground[1] =  1;
-  ground[2] =  0;
-  ground[3] = -100.1;
-  ShadowMatrix(ground,light,slope_shadow_matrix);
-
-  ground[0] = 0;
-  ground[1] = 1;
-  ground[2] = 0;
-  ground[3] = -100.1;
-  ShadowMatrix(ground,light,top_shadow_matrix);
-
-  glClearColor(0.0,0.0,0.0,1.0);
-  if (boFlatShading)
-  {
-    glShadeModel(GL_FLAT);
-    printf("Using flat shading.\n");
-  }
-  glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
-  glMaterialfv(GL_FRONT,GL_SPECULAR,mat_specular);
-  glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_diffuse);
-  glLightfv(GL_LIGHT0,GL_POSITION,light_position);
-  glLightfv(GL_LIGHT0,GL_DIFFUSE,white_light);
-  glLightfv(GL_LIGHT0,GL_SPECULAR,white_light);
-
-  glEnable(GL_DEPTH_TEST);
-  glEnable(GL_LIGHTING);
-  glEnable(GL_LIGHT0);
 }
 
 
 void adjust_zoom(float field_of_view)
 {
+  float aspect = (GLfloat)window_xsize/(GLfloat)window_ysize;
+  float near_dist = 1.0;
+  float far_dist = 10000.0;
+  
   glViewport(0,0,(GLsizei)window_xsize,(GLsizei)window_ysize);
+
+  // setup perspective for non-SSG rendering
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
-  gluPerspective(field_of_view,
-                 (GLfloat)window_xsize/(GLfloat)window_ysize,3,9500.0);
+  gluPerspective(field_of_view/aspect,aspect,near_dist,far_dist);
   glMatrixMode(GL_MODELVIEW);
+
+  // setup perspective for SSG rendering
+  context->setFOV(field_of_view,field_of_view/aspect);
+  context->setNearFar(near_dist,far_dist);
+}
+
+
+void adjust_model_zoom(float size_ratio, float field_of_view, float distance)
+{
+  int model_window_xsize = window_xsize*size_ratio;
+  int model_window_ysize = window_ysize*size_ratio;
+  float aspect = (GLfloat)model_window_xsize/(GLfloat)model_window_ysize;
+  float near_dist = distance - 50.0;
+  near_dist = near_dist > 1.0 ? near_dist : 1.0;
+  float far_dist = near_dist + 100.0;
   
-  context->setFOV(field_of_view,field_of_view * window_ysize/ window_xsize);
+  glViewport(0,(GLsizei)(window_ysize-model_window_ysize),
+      (GLsizei)model_window_xsize,(GLsizei)model_window_ysize);
+  glScissor(0,(GLsizei)(window_ysize-model_window_ysize),
+      (GLsizei)model_window_xsize,(GLsizei)model_window_ysize);
+  glEnable(GL_SCISSOR_TEST);
+
+  // setup perspective for non-SSG rendering
+  glMatrixMode(GL_PROJECTION);
+  glLoadIdentity();
+  gluPerspective(field_of_view/aspect,aspect,near_dist,far_dist);
+  glMatrixMode(GL_MODELVIEW);
+
+  // setup perspective for SSG rendering
+  context->setFOV(field_of_view,field_of_view/aspect);
+  context->setNearFar(near_dist,far_dist);
 }
 
 
@@ -602,20 +549,6 @@ void oscillo()
 }
 
 
-void makeShadowMatrix(sgMat4 sm, float x, float y)
-{
-  sgVec4  plane, light;
-  (void)Global::scenery->getHeightAndPlane(x, y, plane);
-
-  light[0] = lightposn[0];
-  light[1] = lightposn[1];
-  light[2] = lightposn[2];
-  light[3] = 0;
-
-  sgNormaliseVec4(light);
-  ShadowMatrix(plane, light, sm);
-}
-
 /**
  * Converts a vector from FDM to graphics representation
  */
@@ -625,6 +558,15 @@ CRRCMath::Vector3 FDM2Graphics(CRRCMath::Vector3 const& v)
 }
 
 
+/**
+ * Converts a vector from graphics to FDM representation
+ */
+CRRCMath::Vector3 Graphics2FDM(CRRCMath::Vector3 const& v)
+{
+  return CRRCMath::Vector3(-v.r[2], v.r[0], -v.r[1]);
+}
+
+
 /*****************************************************************************/
 /** \brief The per-frame OpenGL display routine
  *
@@ -639,83 +581,188 @@ void display()
   // Prepare the current frame buffer and reset
   // the modelview matrix (for non-SSG drawing)
   GLbitfield clearmask = GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT;
-   if (vidbits.stencil)
-    {
+  if (vidbits.stencil)
+  {
     clearmask |= GL_STENCIL_BUFFER_BIT;
-    }
+  }
   glClear(clearmask);
 
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
-  glPushMatrix();
   //~ glBlendFunc(GL_ONE, GL_ZERO);
   //~ glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
 
-  // Set up the viewing transformation (for SSG drawing)
-  sgVec3 viewpos, planepos, up;
-  
-  sgSetVec3(viewpos, player_pos.r[0], player_pos.r[1], player_pos.r[2]);                
-  sgSetVec3(planepos, looking_pos.r[0], looking_pos.r[1], looking_pos.r[2]);
-  sgSetVec3(up, 0.0, 1.0, 0.0);
-  context->setCameraLookAt(viewpos, planepos, up);
-
   // 3D scene
-  if( Global::scenery != NULL)
+  if (Global::scenery->getID() != 0)
   {
-    // 3D scene: sky sphere
-    draw_sky(&viewpos, Global::Simulation->getTotalTime());
+    glPushMatrix();
+
+    // Set up the viewing transformation (for SSG drawing)
+    sgVec3 vPlayerPos, vLookingPos, vUp;
+    sgSetVec3(vPlayerPos, player_pos.r[0], player_pos.r[1], player_pos.r[2]);                
+    sgSetVec3(vLookingPos, looking_pos.r[0], looking_pos.r[1], looking_pos.r[2]);
+    sgSetVec3(vUp, 0.0, 1.0, 0.0);  
+    context->setCameraLookAt(vPlayerPos, vLookingPos, vUp);
+
+    // Set up ambient light and sun light for SSG rendering,
+    // according to scenery's sky definition
+    sgVec4 ambient_light;
+    Global::scenery->getAmbientColor(ambient_light);
+    sgVec4 sun_light;
+    Global::scenery->getSunColor(sun_light);
+    sgVec3 sun_pos;
+    Global::scenery->getSunPosition(sun_pos);
+    ssgGetLight(0)->setColour(GL_AMBIENT, ambient_light);
+    ssgGetLight(0)->setColour(GL_DIFFUSE, sun_light);
+    ssgGetLight(0)->setColour(GL_SPECULAR, sun_light);
+    ssgGetLight(0)->setPosition(sun_pos);
+
+    // Set up the fog, according to scenery's sky definition
+    float fog = Global::scenery->getFogDensity();
+    sgVec4 fog_color;
+    Global::scenery->getFogColor(fog_color);
+    glHint(GL_FOG_HINT, GL_NICEST);   //set the fog to look nicest
+    glFogi(GL_FOG_MODE, GL_EXP2);     //set the fog mode
+    glFogfv(GL_FOG_COLOR, fog_color);
+    glFogf(GL_FOG_DENSITY, fog);
+
+    // 3D scene: sky
+    glDisable(GL_FOG);
+    draw_sky(&vPlayerPos, Global::Simulation->getSimulationTimeSinceReset());
+
+    // only enable fog after sky has been rendered
+    if (fog > 0)
+      glEnable(GL_FOG);
+    
+    // 3D scene: scenery
+    // This shall be drawn first so that transparent terrain
+    // (used in model-based photo realistic sceneries) hides
+    // the objects drawn successively (e.g. airplane)
+    Global::scenery->draw(Global::Simulation->getSimulationTimeSinceReset());
   
-    // 3D scene: airplane
-    if (Global::aircraft->getModel() != NULL)
+    // 3D scene: airplanes
+    // Draw all airplane models, including robots, and their shadows
+    draw_all_airplanes(Global::Simulation->getSimulationTimeSinceReset());
+    
+    // 3D scene: casted shadows (by all objects in the scene)    
+#if (SHADOW_TYPE==SHADOW_VOLUME)
+    // mark casted shadow in thr stencil buffer
+    Global::scenery->draw_shadows(Global::Simulation->getSimulationTimeSinceReset());
+    draw_all_airplanes(Global::Simulation->getSimulationTimeSinceReset(), false);
+    
+    // draw casted shadows marked in the stencil buffer
+    shadowVolumeDrawShadows();
+#endif
+
+    // ssgCullAndDraw() ends up with an identity modelview matrix,
+    // so we have to set up our own viewing transformation to
+    // properly rendere following optional items.
+    // NB: light & fog are left unchanged, so they are still ok.
+    gluLookAt(player_pos.r[0], player_pos.r[1], player_pos.r[2],
+              looking_pos.r[0], looking_pos.r[1], looking_pos.r[2],
+              0.0, 1.0, 0.0);
+    context->forceBasicState();
+    
+    // 3D scene: game-mode-specific stuff (pylons etc.)
+    Global::gameHandler->draw();
+
+    // disable the fog when finished rendering the scene
+    glDisable(GL_FOG);
+
+    // 3D scene: optionally draw thermals
+    if (Global::training_mode == TRUE)
     {
-      // For SSG rendering, this call does not draw anything,
-      // but calculates the airplane's transformation matrix
-      glDisable(GL_TEXTURE_2D);
-      glEnable(GL_LIGHTING);
-      glEnable(GL_LIGHT0);
-      Global::aircraft->getModel()->draw(Global::aircraft->getFDM());
+      draw_thermals(Global::aircraft->getPos());
+    }
+    
+    // 3D scene: optionally draw wind vectors
+    if (Global::windVectors > 0)
+    {
+      Global::scenery->drawWindField(Global::aircraft->getPos(), Global::windVectors);
     }
   
-    // 3D scene: scenery
-    Global::scenery->draw(Global::Simulation->getTotalTime());
-  }
-  // Lighting setup. Only needed as long as there are
-  // non-SSG parts that modify the light sources.
-  sgVec4 lightamb;
-  sgSetVec4(lightamb , 0.2f, 0.2f, 0.2f, 1.0f);
-  ssgGetLight(0)->setPosition(lightposn);
-  ssgGetLight(0)->setColour(GL_AMBIENT, lightamb);
+    // 3D scene: optionally draw model-view window
+    if (!Global::testmode && Global::modelView > 0)
+    {
+      // get half the aircraft size and aircraft position
+      float aircraftSize = Global::aircraft->getModel()->getAircraftSize();
+      CRRCMath::Vector3 aircraft_pos = FDM2Graphics(Global::aircraft->getPos());
+      
+      // compute model-view field of view    
+      // remember: tan(fieldOfView/2) * distance = aircraftSize
+      float distance_to_model = (aircraft_pos - player_pos).length();
+      float modelFieldOfView = 180.0/M_PI*2.*atan(aircraftSize/distance_to_model);
+      modelFieldOfView *= (1.0 + MODEL_VIEW_FOV_MARGIN);
+      
+      float fieldOfView = zoom_get();
+      float modelViewSize = MODEL_VIEW_SIZE;
+      float fov_ratio = fieldOfView*modelViewSize/modelFieldOfView;
+      float fov_ratio_limit = 1.0;
+      
+      // draw model-view window if required or convenient (in auto mode)
+      if (Global::modelView == 1 || fov_ratio > fov_ratio_limit)
+      {
+        glPushAttrib(GL_VIEWPORT_BIT);
+        glPushAttrib(GL_ENABLE_BIT | GL_TRANSFORM_BIT | GL_LIGHTING_BIT);
+        
+        // set viewport  
+        adjust_model_zoom(modelViewSize, modelFieldOfView, distance_to_model);
 
-  // Draw the scenegraph (airplane model, shadow)
-  ssgCullAndDraw(scene);
-  context->forceBasicState();
+        // fill the viewport with a translucent white bkg
+        glMatrixMode(GL_PROJECTION);
+        glPushMatrix(); 
+        glLoadIdentity();
+        gluOrtho2D(0.0,1.0,0.0,1.0);
+        glMatrixMode(GL_MODELVIEW);
+        glPushMatrix();
+        glLoadIdentity();
+        glDisable(GL_LIGHTING);
+        glDisable(GL_TEXTURE_2D);
+        glDisable(GL_DEPTH_TEST);
+        glEnable(GL_BLEND);
+        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+        glBegin(GL_QUADS);
+        glColor4f(1.0,1.0,1.0,0.5);
+        glVertex2f(0.0,0.0);
+        glVertex2f(1.0,0.0);
+        glVertex2f(1.0,1.0);
+        glVertex2f(0.0,1.0);
+        glEnd();      
+        
+        // restore original settings but for viewport
+        glPopAttrib();
+        
+        // Set up the viewing transformation (for SSG drawing)
+        // camera looking straight at the model
+        sgVec3 vPlanePos;
+        sgSetVec3(vPlanePos, plane_pos.r[0], plane_pos.r[1], plane_pos.r[2]);                
+        context->setCameraLookAt(vPlayerPos, vPlanePos, vUp);
+              
+        // clear zbuffer in the close-view window
+        glClearDepth(1.0);
+        glClear(GL_DEPTH_BUFFER_BIT);
+         
+        // draw the airplane model, without any casted shadow
+        Global::aircraft->getModel()->draw();
 
-  // ssgCullAndDraw() ends up with an identity modelview matrix,
-  // so we have to set up our own viewing transformation for
-  // the thermals
-  glLoadIdentity();
-  gluLookAt(player_pos.r[0], player_pos.r[1], player_pos.r[2],
-            looking_pos.r[0], looking_pos.r[1], looking_pos.r[2],
-            0.0, 1.0, 0.0);
+        // restore original viewport and camera
+        glPopAttrib();
+        glMatrixMode(GL_PROJECTION);
+        glPopMatrix(); 
+        glMatrixMode(GL_MODELVIEW);
+        glPopMatrix();
+
+        context->forceBasicState();
+      }
+    }
   
-  if (Global::training_mode==TRUE)
-  {
-    draw_thermals(Global::aircraft->getPos());
+    glPopMatrix();
   }
-  
-  // 3D scene: game-mode-specific stuff (pylons etc.)
-  Global::gameHandler->draw();
-
-  glPopMatrix();
-
-
-  // Overlay: game handler
-  Global::gameHandler->display_infos(window_xsize, window_ysize);
 
   // Overlay: scope for audio interface
-  if ( Global::testmode.test_mode
+  if (Global::testmode
        &&
-       (Global::TXInterface->inputMethod() == T_TX_Interface::eIM_audio) )
+       (Global::TXInterface->inputMethod() == T_TX_Interface::eIM_audio))
   {
     GlOverlay::setupRenderingState(window_xsize, window_ysize);
     oscillo();
@@ -766,9 +813,11 @@ void display()
     glMatrixMode(GL_MODELVIEW);
 #endif
     GlOverlay::restoreRenderingState();
-
   }
 
+  // Overlay: game handler
+  Global::gameHandler->display_infos(window_xsize, window_ysize);
+
   // Overlay: console
   console->render(window_xsize, window_ysize);
   
@@ -899,13 +948,20 @@ std::string GetVideoInfoString(const char *indent)
 }
 
 
-SDL_Surface* video_setup_colordepth(int& nX, int& nY, int& nFullscreen, int color_depth)
+SDL_Surface* video_setup_colordepth(int& nX, int& nY, int& nFullscreen, int color_depth, int nMultisamples)
 {
   SDL_Surface* screen;
 
   int cbits = (color_depth <= 16) ?  5 :  8;
   int zbits = (color_depth <= 16) ? 16 : 24;
-
+  
+  if (nMultisamples > 0)
+  {
+    cbits =  8;
+    if (zbits == 24)
+      zbits = 32;
+  }
+  
   /* setup GL attributes */
   SDL_GL_SetAttribute(SDL_GL_RED_SIZE, cbits);
   SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, cbits);
@@ -914,7 +970,26 @@ SDL_Surface* video_setup_colordepth(int& nX, int& nY, int& nFullscreen, int colo
   SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE,1);
   SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, zbits);
   SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
-
+  
+  if (nMultisamples > 0)
+  {
+    SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);
+    SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, nMultisamples);
+    glEnable(GL_MULTISAMPLE);
+  
+    // http://adrianboeing.blogspot.de/2010/01/antialiasing-multisampling-opengl-with.html
+    glHint(GL_LINE_SMOOTH_HINT, GL_FASTEST);
+    glHint(GL_POLYGON_SMOOTH_HINT, GL_FASTEST);
+    glEnable(GL_LINE_SMOOTH);
+    glEnable(GL_POLYGON_SMOOTH);
+  }
+  else
+  {
+    SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 0);
+    SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, 0);
+    glDisable(GL_MULTISAMPLE);
+  }
+  
   // Load defaults?
   if (nX < 100 || nY < 100)
   {
@@ -990,9 +1065,10 @@ int setupScreen(int nX, int nY, int nFullscreen)
     printf("Screen resolution : %d x %d \n",screen_xsize, screen_ysize);
   }
   int color_depth = cfgfile->getInt("video.color_depth", 24);
-
+  int nMultisamples = cfgfile->getInt("video.multisamples", 1); // anti aliasing: 0 (off), 1, 2, 4, 8, 16
+    
   SDL_video_flags = SDL_OPENGL;
-#ifdef linux
+#if defined linux || defined WIN32
   SDL_video_flags |= SDL_RESIZABLE;
 #endif
   if((nX==0)&&(nY==0)&&(nFullscreen==1))//resolution auto
@@ -1001,22 +1077,34 @@ int setupScreen(int nX, int nY, int nFullscreen)
     nY = screen_ysize;
     resolution_auto = 1;
   }
-  SDL_Surface* screen = video_setup_colordepth(nX, nY, nFullscreen, color_depth);
+  SDL_Surface* screen = video_setup_colordepth(nX, nY, nFullscreen, color_depth, nMultisamples);
 
   if (!screen)
   {
     color_depth = 32;
-    screen = video_setup_colordepth(nX, nY, nFullscreen, color_depth);
+    screen = video_setup_colordepth(nX, nY, nFullscreen, color_depth, nMultisamples);
   }
   if (!screen)
   {
     color_depth = 16;
-    screen = video_setup_colordepth(nX, nY, nFullscreen, color_depth);
+    screen = video_setup_colordepth(nX, nY, nFullscreen, color_depth, nMultisamples);
+  }
+  if (!screen && nMultisamples > 0)
+  {
+    color_depth = 32;
+    nMultisamples = 0;
+    screen = video_setup_colordepth(nX, nY, nFullscreen, color_depth, nMultisamples);
+  }
+  if (!screen && nMultisamples > 0)
+  {
+    color_depth = 16;
+    nMultisamples = 0;
+    screen = video_setup_colordepth(nX, nY, nFullscreen, color_depth, nMultisamples);
   }
   if (!screen)
   {
     color_depth = 8;
-    screen = video_setup_colordepth(nX, nY, nFullscreen, color_depth);
+    screen = video_setup_colordepth(nX, nY, nFullscreen, color_depth, nMultisamples);
   }
 
   if (!screen)
@@ -1034,10 +1122,10 @@ int setupScreen(int nX, int nY, int nFullscreen)
 
   // Write back to config.
   if (resolution_auto ) 
-    {
+  {
     nX = 0;
     nY = 0;
-    }
+  }
   if (nFullscreen)
   {
     cfgfile->setAttributeOverwrite("video.fullscreen.fUse", "1");
@@ -1051,6 +1139,7 @@ int setupScreen(int nX, int nY, int nFullscreen)
     cfgfile->setAttributeOverwrite("video.resolution.window.y", nY);
   }
   cfgfile->setAttributeOverwrite("video.color_depth", color_depth);
+  cfgfile->setAttributeOverwrite("video.multisamples", nMultisamples);
   
   window_xsize = screen->w;
   window_ysize = screen->h;
@@ -1107,14 +1196,17 @@ void reshape(int w, int h)
  */
 void resize_window(int w, int h)
 {
-#ifdef linux
+#if defined linux || defined WIN32
   SDL_Surface *screen;
-  SDL_video_flags &= ~SDL_FULLSCREEN;
-  screen = SDL_SetVideoMode(w, h,
-                            cfgfile->getInt("video.color_depth"),
-                            SDL_video_flags);
-  if (screen)
-    reshape(screen->w, screen->h);
+  if(!(SDL_video_flags&SDL_FULLSCREEN)) // for WinXP
+  {
+    //SDL_video_flags &= ~SDL_FULLSCREEN;
+    screen = SDL_SetVideoMode(w, h,
+                              cfgfile->getInt("video.color_depth"),
+                              SDL_video_flags);
+    if (screen)
+      reshape(screen->w, screen->h);
+  }
 #else
   // until we find a solution, just ignore the event
   // --------------------------------------------------
@@ -1134,7 +1226,8 @@ void cleanup()
 
 void read_config(SimpleXMLTransfer* cf)
 {
-  flSloppyCam = cf->getDouble("video.camera.sloppy", 0.0);
+  flSmartCam = cf->getDouble("video.camera.smart", 0);
+  flSloppyCam = cf->getDouble("video.camera.sloppy", 0);
 }
 
 
@@ -1203,38 +1296,151 @@ void drawSolidCube(GLfloat size)
 
 /* end modified GLUT code */
 
+
+void InitSmartCamera()
+{
+  flSmartCam_time = 0.;
+
+  CRRCMath::Vector3 plane_pos = FDM2Graphics(Global::aircraft->getPos());
+  CRRCMath::Vector3 plane_dir = plane_pos - player_pos;
+  CRRCMath::Vector3 plane_dir_FDM = Graphics2FDM(plane_dir);
+  flSmartCam_center_w = atan2(plane_dir_FDM.r[1], plane_dir_FDM.r[0]); 
+  
+  double dist = sqrt(plane_dir_FDM.r[0]*plane_dir_FDM.r[0] + plane_dir_FDM.r[1]*plane_dir_FDM.r[1]) + 1.;
+  flSmartCam_center_h = atan(plane_dir_FDM.r[2]/dist);
+  
+  flSmartCam_var_w = 0.;
+  flSmartCam_var_h = 0.;
+}
+
+
 void UpdateCamera(float flDeltaT)
 {
-  double  phimax    = flSloppyCam*zoom_get();
-  double  max       = cos(phimax)*cos(phimax);
+  double fieldOfViewRad = M_PI/180.0*zoom_get();
   
+  double pi_two  = 2.*M_PI;
+  double pi_half = 0.5*M_PI;
   CRRCMath::Vector3 plane_pos = FDM2Graphics(Global::aircraft->getPos());
-  CRRCMath::Vector3 look_dir  = looking_pos - player_pos;
   CRRCMath::Vector3 plane_dir = plane_pos - player_pos;
-  
-  if (plane_dir.angle_cos_sqr(look_dir) < max)
+  CRRCMath::Vector3 plane_dir_FDM = Graphics2FDM(plane_dir);
+  double phi_w = atan2(plane_dir_FDM.r[1], plane_dir_FDM.r[0]);  
+  double dist = sqrt(plane_dir_FDM.r[0]*plane_dir_FDM.r[0] + plane_dir_FDM.r[1]*plane_dir_FDM.r[1]) + 1.;
+  double phi_h = atan(plane_dir_FDM.r[2]/dist);
+
+  if (!Global::testmode)
   {
-    // Adjust the length of look_dir so that plane_dir, look_dir and
-    // (look_dir - plane_dir) form a right angle triangle.
-    double k = (plane_dir.inner(plane_dir)) / (plane_dir.inner(look_dir));
-    look_dir = look_dir * k;
-    // Calculate the difference vector...
-    CRRCMath::Vector3 diff_dir = look_dir - plane_dir;
-    double ddl = diff_dir.length();
-    if (ddl > 0.001)
+    // update smart camera automatic center/range definition
+    flSmartCam_time += flDeltaT;
+    flSmartCam_time = flSmartCam_time > SMARTCAM_TIME ? SMARTCAM_TIME : flSmartCam_time;
+    if (flSmartCam_time > 0.)
     {
-      // ...and adjust its length (the right angle triangle remains!) so that
-      // the angle between look_dir and plane_dir equals phimax.
-      double tan_phi = fabs(tan(acos(sqrt(max))));
-      look_dir = plane_dir + diff_dir*(plane_dir.length()*tan_phi/ddl);
-      looking_pos = look_dir + player_pos;
+      double w = flDeltaT/flSmartCam_time;
+      double w1 = 1. - w;
+
+      double dphi;
+
+      dphi = phi_w - flSmartCam_center_w; 
+      if (fabs(dphi) > M_PI)
+        dphi = dphi > 0. ? dphi - pi_two : dphi + pi_two;
+      flSmartCam_center_w += w*dphi;
+      if (fabs(flSmartCam_center_w) > M_PI)
+        flSmartCam_center_w = flSmartCam_center_w > 0. ? flSmartCam_center_w - pi_two : flSmartCam_center_w + pi_two;
+      flSmartCam_var_w = w1*flSmartCam_var_w + w*dphi*dphi;
+        
+      dphi = phi_h - flSmartCam_center_h;
+      flSmartCam_center_h += w*dphi;
+      if (fabs(flSmartCam_center_h) > pi_half)
+        flSmartCam_center_h = flSmartCam_center_h > 0. ? pi_half : -pi_half;
+      flSmartCam_var_h = w1*flSmartCam_var_h + w*dphi*dphi;
     }
   }
+  
+  if (flSmartCam > 0 && !Global::testmode) // smart camera option active
+  {  
+    float phi_w0, phi_h0, range_w, range_h;
+
+    // try to get game-specific parameters to drive smart camera
+    // If not available use automatic center/range definition
+    if (!Global::gameHandler->GetSmartCameraPar(phi_w0, phi_h0, range_w, range_h))
+    {
+      phi_w0 = flSmartCam_center_w;
+      phi_h0 = flSmartCam_center_h;
+      range_w = SMARTCAM_RANGE_FAC*sqrt(flSmartCam_var_w);
+      range_h = SMARTCAM_RANGE_FAC*sqrt(flSmartCam_var_h);
+    }
+    
+    // always use full movement range in height
+    // but only used specified range in width
+    CRRCMath::Vector3 look_dir_FDM(plane_dir_FDM);
+    double dphimax_w0 = 0.5*fieldOfViewRad*SMARTCAM_MAX_FRACTION*flSmartCam;
+    double dphimax_h0 = 0.5*fieldOfViewRad*SMARTCAM_MAX_FRACTION*window_ysize/window_xsize;
+    range_w = range_w > dphimax_w0 ? range_w : dphimax_w0;
+    range_h = range_h > dphimax_h0 ? range_h : dphimax_h0;
+    
+    // set look_dir orientation
+    {
+      double dphi = phi_w - phi_w0; 
+      if (fabs(dphi) > M_PI)
+        dphi = dphi > 0. ? dphi - pi_two : dphi + pi_two;
+      double sign = dphi >= 0. ? 1. : -1.;
+      if (fabs(dphi) < range_w)
+        dphi = sign*dphimax_w0*sin(pi_half*fabs(dphi)/range_w);
+      else
+        dphi = sign*dphimax_w0*sin(pi_half*(M_PI - fabs(dphi))/(M_PI - range_w));
+
+      // rotate plane_dir_FDM towards look_dir_FDM along Z axis 
+      double cc = cos(dphi);
+      double ss = sin(dphi);
+      look_dir_FDM.r[0] =  cc*plane_dir_FDM.r[0] + ss*plane_dir_FDM.r[1];
+      look_dir_FDM.r[1] = -ss*plane_dir_FDM.r[0] + cc*plane_dir_FDM.r[1];
+    }
+
+    // set look_dir elevation
+    {
+      double dphi = phi_h - phi_h0;    
+      double sign = dphi >= 0. ? 1. : -1.;
+      if (fabs(dphi) < range_h)
+        dphi = sign*dphimax_h0*sin(pi_half*fabs(dphi)/range_h);
+      else
+        dphi = sign*dphimax_h0;
+      look_dir_FDM.r[2] = dist*tan(phi_h - dphi);
+    }
+    
+    // set looking position
+    looking_pos = player_pos + FDM2Graphics(look_dir_FDM);
+  }
   else
   {
-    // slowly follow model
-    CRRCMath::Vector3 diff_pos = plane_pos - looking_pos;
-    looking_pos = looking_pos + diff_pos*(0.02*flDeltaT);
+    double  phimax   = flSloppyCam*fieldOfViewRad;
+    double  max      = cos(phimax)*cos(phimax);
+    
+    CRRCMath::Vector3 look_dir  = looking_pos - player_pos;
+    CRRCMath::Vector3 plane_dir = plane_pos - player_pos;
+    
+    if (plane_dir.angle_cos_sqr(look_dir) < max)
+    {
+      // Adjust the length of look_dir so that plane_dir, look_dir and
+      // (look_dir - plane_dir) form a right angle triangle.
+      double k = (plane_dir.inner(plane_dir)) / (plane_dir.inner(look_dir));
+      look_dir = look_dir * k;
+      // Calculate the difference vector...
+      CRRCMath::Vector3 diff_dir = look_dir - plane_dir;
+      double ddl = diff_dir.length();
+      if (ddl > 0.001)
+      {
+        // ...and adjust its length (the right angle triangle remains!) so that
+        // the angle between look_dir and plane_dir equals phimax.
+        double tan_phi = fabs(tan(acos(sqrt(max))));
+        look_dir = plane_dir + diff_dir*(plane_dir.length()*tan_phi/ddl);
+        looking_pos = look_dir + player_pos;
+      }
+    }
+    else
+    {
+      // slowly follow model
+      CRRCMath::Vector3 diff_pos = plane_pos - looking_pos;
+      looking_pos = looking_pos + diff_pos*(0.02*flDeltaT);
+    }
   }
 }
 
@@ -1250,6 +1456,23 @@ void getWindowSize(int& x, int& y)
   y = window_ysize;
 }
 
+/**
+ * Read the "smart cam" setting from mod_video
+ */
+float getSmartCam()
+{
+  return flSmartCam;
+}
+
+/**
+ * Write the "smart cam" setting
+ * \param flValue  New value for smart cam
+ */
+void setSmartCam(float flValue)
+{
+  flSmartCam = flValue;
+}
+
 /**
  * Read the "sloppy cam" setting from mod_video
  */
