@@ -30,15 +30,32 @@
 
 #include "crrc_sky.h"
 #include "crrc_graphics.h"
-
 #include "../mod_misc/filesystools.h"
 
+// Default ambient light settings
+#define DEFAULT_AMBIENT_R     (0.5)
+#define DEFAULT_AMBIENT_G     (0.5)
+#define DEFAULT_AMBIENT_B     (0.5)
+
+// Default sun settings
+#define DEFAULT_SUN_LENSFLARE (1)
+#define DEFAULT_SUN_AZIMUTH   (210)
+#define DEFAULT_SUN_ELEVATION (25)
+#define DEFAULT_SUN_R         (1.0)
+#define DEFAULT_SUN_G         (1.0)
+#define DEFAULT_SUN_B         (1.0)
+
+// Default fog settings
+#define DEFAULT_FOG_DENSITY   (0.2)
+#define DEFAULT_FOG_R         (0.85)
+#define DEFAULT_FOG_G         (0.90)
+#define DEFAULT_FOG_B         (0.95)
 
 namespace Video
 {
-
 /** A pointer to the current sky visualization */
-static SkyRenderer        *theSky = NULL;
+static SkyRenderer        *sky = NULL;
+
 /** A copy of the sky parameters (to be able to recreate the visualization
  *  without reloading the scenery file).
  */
@@ -59,6 +76,11 @@ static const char *boxtex_attribs[] =
   "down.filename"
 };
 
+/**
+ *  sun flare effect data
+ */
+static ssgTransform  *sun = NULL;
+static ssgaLensFlare *sunFlare = NULL;
 
 /**
  *  Setup the sky from the given XML configuration section.
@@ -67,13 +89,21 @@ static const char *boxtex_attribs[] =
  *  and additional config file parameters. An existing sky
  *  object wil be deleted.
  *
- *  \param xml  XML configuration section for the sky renderer object
- *              (pass NULL to reload the existing sky)
+ *  \param xml      XML configuration section for the sky renderer object
+ *                  (pass NULL to reload the existing sky)
  *  \return true for success
  */
-bool setup_sky(SimpleXMLTransfer *xml)
+SkyRenderer* setup_sky(SimpleXMLTransfer *xml)
 {
-  bool boReturn = true;
+  // creat a sun flare effect if it does not already exist
+  if (sunFlare == NULL)
+  {
+    sunFlare = new ssgaLensFlare();
+    sun = new ssgTransform();
+    sun->addKid(sunFlare);
+    // make it permanent
+    sun->ref();
+  }
   
   // create sky from scratch or reload?
   if (xml != NULL)
@@ -86,21 +116,17 @@ bool setup_sky(SimpleXMLTransfer *xml)
     sky_par = new SimpleXMLTransfer(xml);
   }
   
-  if (sky_par == NULL)
-  {
-    boReturn = false;
-  }
-  else
+  if (sky_par != NULL)
   {
     // delete an existing sky renderer object
-    if (theSky != NULL)
+    if (sky != NULL)
     {
-      delete theSky;
-      theSky = NULL;
+      delete sky;
+      sky = NULL;
     }
 
     std::string sky_type = sky_par->attribute("type", "original");
-
+    
     try
     {
       if (sky_type == "original")
@@ -113,7 +139,7 @@ bool setup_sky(SimpleXMLTransfer *xml)
           // find full path for the texture
           texture = FileSysTools::getDataPath(texture);
         }
-        theSky = new CRRCSkyDome(texture.c_str(), radius);
+        sky = new CRRCSkyDome(sky_par, texture.c_str(), radius);
       }
       else if (sky_type == "box")
       {
@@ -140,7 +166,7 @@ bool setup_sky(SimpleXMLTransfer *xml)
         // than the scenery itself)
         float texoffset = cfgfile->getDouble("video.skybox.texture_offset", 0.0);
 
-        theSky = new SkyBox((const char**)textures, size, texoffset);
+        sky = new SkyBox(sky_par, (const char**)textures, size, texoffset);
 
         for (int i = 0; i < 6; i++)
         {
@@ -157,13 +183,13 @@ bool setup_sky(SimpleXMLTransfer *xml)
           // find full path for the texture
           texture = FileSysTools::getDataPath(texture);
         }
-        theSky = new CRRCPanoDome(texture.c_str(), radius);
+        sky = new CRRCPanoDome(sky_par, texture.c_str(), radius);
       }
       else
       {
         // unknown type, create a default CRRCSkyDome
         fprintf(stderr, "Unknown sky type \"%s\" in scenery file, using default.\n", sky_type.c_str());
-        theSky = new CRRCSkyDome(FileSysTools::getDataPath("textures/clouds.rgb").c_str(),
+        sky = new CRRCSkyDome(sky_par, FileSysTools::getDataPath("textures/clouds.rgb").c_str(),
                                  8000.0);
       }
     }
@@ -172,10 +198,9 @@ bool setup_sky(SimpleXMLTransfer *xml)
       std::string s = "Unable to create sky visualization:\n";
       s += e.what();
       std::cerr << s << std::endl;
-      boReturn = false;
     }
   }
-  return boReturn;
+  return sky;
 }
 
 
@@ -184,25 +209,94 @@ bool setup_sky(SimpleXMLTransfer *xml)
  */
 void draw_sky(sgVec3 *campos, double dt)
 {
-  if (theSky != NULL)
+  if (sky != NULL)
   {
-    theSky->update(campos, dt);
-    theSky->preDraw();
+    sky->update(campos, dt);
+    sky->preDraw();
   }
 }
 
+
 /**
- * Deallocate all sky ressources
+ * Deallocate all sky resources
  */
 void cleanup_sky()
 {
-  delete theSky;
-  theSky = NULL;
+  ssgDeRefDelete(sun);
+  delete sky;
+  sky = NULL;
   delete sky_par;
   sky_par = NULL;
 }
 
 
+/**
+ *  This constructor reads all common skybox parameters
+ *  from the xml description file.
+**/
+SkyRenderer::SkyRenderer(SimpleXMLTransfer *xml)
+{
+  // read and set ambient light color
+  {
+    SimpleXMLTransfer *amb = xml->getChild("ambient", true);
+    float r, g, b;
+        
+    r = amb->getDouble("color.r", DEFAULT_AMBIENT_R);   
+    g = amb->getDouble("color.g", DEFAULT_AMBIENT_G);   
+    b = amb->getDouble("color.b", DEFAULT_AMBIENT_B);   
+    sgSetVec4(vSkyAmbientColor, r, g, b, 1.0);    
+  }       
+      
+  // read and set sun position and light color
+  {
+    SimpleXMLTransfer *sun = xml->getChild("sun", true);
+    float azm, ele, x, y, z, r, g, b;
+    
+    // get sun azimuth and elevation angle in degrees
+    // azimuth is 0 north, 90 east.
+    // Default are set according to standard skybox images
+    azm = sun->getDouble("position.azimuth", DEFAULT_SUN_AZIMUTH);      
+    ele = sun->getDouble("position.elevation", DEFAULT_SUN_ELEVATION);   
+
+    // convert azimuth and elevation into (opengl) 
+    // scene reference system where:
+    //   +Z is S
+    //   +X is E
+    //   +Y is up
+    azm = SG_DEGREES_TO_RADIANS*(azm - 90.);
+    ele = SG_DEGREES_TO_RADIANS*ele;
+    y = sin(ele);
+    x = cos(ele)*cos(azm);
+    z = cos(ele)*sin(azm);
+    sgSetVec3(vSkySunPosition, x, y, z);
+    
+    r = sun->getDouble("color.r", DEFAULT_SUN_R);   
+    g = sun->getDouble("color.g", DEFAULT_SUN_G);   
+    b = sun->getDouble("color.b", DEFAULT_SUN_B);   
+    sgSetVec4(vSkySunColor, r, g, b, 1.0);    
+  }       
+      
+  // read and set fog density and color
+  {
+    SimpleXMLTransfer *haze = xml->getChild("haze", true);
+    float dens, r, g, b;
+    
+    // get fog (actually haze) density and color.
+    // Default are set according to standard skybox images
+    // fog density from 0 to 1 is mapped into the reasonable
+    // range 0 .. 0.001 (using opengl GL_EXP2 fog mode)
+    dens = haze->getDouble("density", DEFAULT_FOG_DENSITY);
+    dens = dens > 1.0 ? 1.0 : dens < 0.0 ? 0.0 : dens;
+    flSkyFogDensity = 0.001*dens;
+    
+    r = haze->getDouble("color.r", DEFAULT_FOG_R);   
+    g = haze->getDouble("color.g", DEFAULT_FOG_G);   
+    b = haze->getDouble("color.b", DEFAULT_FOG_B);   
+    sgSetVec4(vSkyFogColor, r, g, b, 1.0);
+  }
+}
+
+
 /**
  *  DRAW_MODE can be: 0 (draw two triangles per face)
  *                    1 (draw one quad per face)
@@ -315,13 +409,27 @@ static const int _texels[4][2] =
  *  \param texoffset  The texels of the box faces will be
  *                    offset from the texture border by this amount.
  */
-SkyBox::SkyBox(const char **textures, float size, float texoffset)
-  : skyroot(NULL), skyboxtrans(NULL), boxsize(size / 2.0f)
+SkyBox::SkyBox(SimpleXMLTransfer *xml, const char **textures, float size, float texoffset)
+  : SkyRenderer(xml), skyroot(NULL), skyboxtrans(NULL), boxsize(size / 2.0f)
 {
   skyroot     = new ssgRoot();
   skyboxtrans = new ssgTransform();
   skyroot->addKid(skyboxtrans);
 
+  // get lens flare effect switch, default to true
+  int fUseLensFlare = xml->getInt("sun.fUse_lensFlare", DEFAULT_SUN_LENSFLARE);      
+  
+  // add sun flare effect
+  if (fUseLensFlare)
+  {
+    // NB: for some reason position vector must have "large"
+    // components, not normalised ones
+    sgVec3 sun_pos;
+    sgScaleVec3(sun_pos, vSkySunPosition, 10000);
+    sun->setTransform(sun_pos);
+    skyroot->addKid(sun);
+  }
+
   // the texture coordinates are the same for each face
   #if (DRAW_MODE == 0)
   ssgTexCoordArray  *texco    = new ssgTexCoordArray(6);
@@ -533,22 +641,22 @@ void SkyBox::preDraw()
 
 /* ******************************************************** */
 // Default sky dome colors
-#define DEFAULT_DOME_R    (0.77)
-#define DEFAULT_DOME_G    (0.92)
-#define DEFAULT_DOME_B    (0.99)
+#define DEFAULT_DOME_R         (0.77)
+#define DEFAULT_DOME_G         (0.92)
+#define DEFAULT_DOME_B         (0.99)
 
-#define DEFAULT_FOG_R     (0.9)
-#define DEFAULT_FOG_G     (0.9)
-#define DEFAULT_FOG_B     (0.9)
+#define DEFAULT_DOME_FOG_R     (0.9)
+#define DEFAULT_DOME_FOG_G     (0.9)
+#define DEFAULT_DOME_FOG_B     (0.9)
 
-#define DEFAULT_CLOUD_R   (0.95)
-#define DEFAULT_CLOUD_G   (0.95)
-#define DEFAULT_CLOUD_B   (0.95)
+#define DEFAULT_DOME_CLOUD_R   (0.95)
+#define DEFAULT_DOME_CLOUD_G   (0.95)
+#define DEFAULT_DOME_CLOUD_B   (0.95)
 
 #if 0
 
-SkyDome::SkyDome(float r, sgVec4 domecol, sgVec4 fogcol, sgVec4 cloudcol)
- : theSky(NULL), radius(r)
+SkyDome::SkyDome(SimpleXMLTransfer *xml, float r, sgVec4 domecol, sgVec4 fogcol, sgVec4 cloudcol)
+ : SkyRenderer(xml), theSky(NULL), radius(r)
 {
   theSky = new ssgaSky();
 
@@ -586,9 +694,9 @@ SkyDome::~SkyDome()
 void SkyDome::repaint()
 {
   theSky->repaint(skyDomeCol, skyFogCol, skyCloudCol,
-                  45*SG_DEGREES_TO_RADIANS,
-                  0, &planetsDummy,
-                  0, &starsDummy);
+                   45*SG_DEGREES_TO_RADIANS,
+                   0, &planetsDummy,
+                   0, &starsDummy);
 }
 
 
@@ -622,9 +730,27 @@ void SkyDome::update(sgVec3 *campos, double dt)
  *                       fall back to an untextured sky dome.
  *  \param r Radius of the sky dome.
  */
-CRRCSkyDome::CRRCSkyDome(const char *cloud_texture, float r)
-  : skyroot(NULL), radius(r)
+CRRCSkyDome::CRRCSkyDome(SimpleXMLTransfer *xml, const char *cloud_texture, float r)
+  : SkyRenderer(xml), skyroot(NULL), skyboxtrans(NULL), radius(r)
 {
+  skyroot     = new ssgRoot();
+  skyboxtrans = new ssgTransform();
+  skyroot->addKid(skyboxtrans);
+
+  // get lens flare effect switch, default to true
+  int fUseLensFlare = xml->getInt("sun.fUse_lensFlare", DEFAULT_SUN_LENSFLARE);      
+  
+  // add sun flare effect
+  if (fUseLensFlare)
+  {
+    // NB: for some reason position vector must have "large"
+    // components, not normalised ones
+    sgVec3 sun_pos;
+    sgScaleVec3(sun_pos, vSkySunPosition, 10000);
+    sun->setTransform(sun_pos);
+    skyroot->addKid(sun);
+  }
+
   repaint(cloud_texture);
 }
 
@@ -635,10 +761,41 @@ CRRCSkyDome::CRRCSkyDome(const char *cloud_texture, float r)
  */
 CRRCSkyDome::~CRRCSkyDome()
 {
+  skyroot->removeAllKids();
   delete skyroot;
 }
 
 
+/**
+ *  To give the perfect illusion, the skybox must always
+ *  stay at a fixed position related to the camera.
+ *
+ *  Call this each time the camera's xyz position changes.
+ *
+ *  \param campos Current camera position.
+ *  \param dt     Elapsed time since last update. Not used.
+ */
+void CRRCSkyDome::update(sgVec3 *campos, double dt)
+{
+  skyboxtrans->setTransform(*campos);
+}
+
+
+/**
+ *  The sky dome is drawn without z-buffer writes to make
+ *  it the most distant object in the scenery.
+ *  Call this method each frame before the rest of the scenery
+ *  is drawn. After drawing the sky, the depth buffer will
+ *  be cleared.
+ */
+void CRRCSkyDome::preDraw()
+{
+  glEnable(GL_DEPTH_TEST);
+  ssgCullAndDraw(skyroot);
+  glClear(GL_DEPTH_BUFFER_BIT);
+}
+
+
 /**
  *  Build the vertex tables that represent the sky dome.
  *  This method is the "work-horse" of the class. It's a
@@ -673,7 +830,7 @@ void CRRCSkyDome::repaint(const char *texture)
   ssgVertexArray    *vertices;
   ssgTexCoordArray  *texels;
   ssgColourArray    *colors;
-  ssgSimpleState    *state_tex;
+  ssgSimpleState    *state;
   sgVec3            vertex;
   sgVec2            texel;
 
@@ -703,7 +860,7 @@ void CRRCSkyDome::repaint(const char *texture)
   OUTER_ELEV    = 0;
   BOTTOM_ELEV   = -200;
 
-  // transformation matrix into CRRCsim's coordinate system
+  // transformation matrix from SSG into CRRCsim's coordinate system
   sgMat4 it = {  {1.0,  0.0,  0.0,  0.0},
                  {0.0,  0.0, -1.0,  0.0},
                  {0.0,  1.0,  0.0,  0.0},
@@ -740,6 +897,7 @@ void CRRCSkyDome::repaint(const char *texture)
     outer_texture[i][0]=(outer_vertex[i][0] / diameter) + 0.5;
     outer_texture[i][1]=(outer_vertex[i][2] / diameter) + 0.5;
 
+
     bottom_vertex[i][0] = cos((double)theta) * BOTTOM_RADIUS;
     bottom_vertex[i][2] = sin((double)theta) * BOTTOM_RADIUS;
     bottom_vertex[i][1] = BOTTOM_ELEV;
@@ -747,28 +905,25 @@ void CRRCSkyDome::repaint(const char *texture)
     bottom_texture[i][1]=(bottom_vertex[i][2] / diameter) + 0.5;
   }
   
-  skyroot = new ssgRoot();
-  
-  state_tex = new ssgSimpleState();
-  
+  state = new ssgSimpleState();
   if (use_textures)
   {
     fprintf(stdout, "Creating CRRCSkyDome with texture \"%s\"\n", texture);
-    state_tex->setTexture(texture);
-    state_tex->enable(GL_TEXTURE_2D);
+    state->setTexture(texture);
+    state->enable(GL_TEXTURE_2D);
   }
   else
   {
     // empty string --> no texture
-    state_tex->disable(GL_TEXTURE_2D);
+    state->disable(GL_TEXTURE_2D);
     fprintf(stdout, "Creating untextured CRRCSkyDome\n");
   }
-  state_tex->setShadeModel(GL_SMOOTH);
-  state_tex->enable(GL_COLOR_MATERIAL);
-  state_tex->setColourMaterial(GL_AMBIENT_AND_DIFFUSE);
-  state_tex->disable(GL_BLEND);
-  state_tex->disable(GL_LIGHTING);
-  state_tex->disable(GL_CULL_FACE);
+  state->setShadeModel(GL_SMOOTH);
+  state->enable(GL_COLOR_MATERIAL);
+  state->setColourMaterial(GL_AMBIENT_AND_DIFFUSE);
+  state->disable(GL_BLEND);
+  state->disable(GL_LIGHTING);
+  state->disable(GL_CULL_FACE);
   
   // The top cap
   vertices  = new ssgVertexArray();
@@ -811,11 +966,10 @@ void CRRCSkyDome::repaint(const char *texture)
                              texels,
                              colors);
 
-  table->setState(state_tex);
+  table->setState(state);
   table->transform(it);
   table->recalcBSphere();
-  skyroot->addKid(table);
-
+  skyboxtrans->addKid(table);
   
   // first ring
   vertices  = new ssgVertexArray();
@@ -864,10 +1018,10 @@ void CRRCSkyDome::repaint(const char *texture)
                           texels,
                           colors);
 
-  table->setState(state_tex);
+  table->setState(state);
   table->transform(it);
   table->recalcBSphere();
-  skyroot->addKid(table);
+  skyboxtrans->addKid(table);
   
   // second ring
   vertices  = new ssgVertexArray();
@@ -916,10 +1070,10 @@ void CRRCSkyDome::repaint(const char *texture)
                           texels,
                           colors);
 
-  table->setState(state_tex);
+  table->setState(state);
   table->transform(it);
   table->recalcBSphere();
-  skyroot->addKid(table);
+  skyboxtrans->addKid(table);
 
   // bottom ring
   vertices  = new ssgVertexArray();
@@ -967,24 +1121,10 @@ void CRRCSkyDome::repaint(const char *texture)
                           texels,
                           colors);
 
-  table->setState(state_tex);
+  table->setState(state);
   table->transform(it);
   table->recalcBSphere();
-  skyroot->addKid(table);
-}
-
-/**
- *  The sky dome is drawn without z-buffer writes to make
- *  it the most distant object in the scenery.
- *  Call this method each frame before the rest of the scenery
- *  is drawn. After drawing the sky, the depth buffer will
- *  be cleared.
- */
-void CRRCSkyDome::preDraw()
-{
-  glEnable(GL_DEPTH_TEST);
-  ssgCullAndDraw(skyroot);
-  glClear(GL_DEPTH_BUFFER_BIT);
+  skyboxtrans->addKid(table);
 }
 
 
@@ -997,8 +1137,8 @@ void CRRCSkyDome::preDraw()
  *  \param texture  Name of the panorama texture.
  *  \param r        Radius of the sky dome.
  */
-CRRCPanoDome::CRRCPanoDome(const char *texture, float r)
- : texfilename(texture), radius(r)
+CRRCPanoDome::CRRCPanoDome(SimpleXMLTransfer *xml, const char *texture, float r)
+ : SkyRenderer(xml), texfilename(texture), radius(r)
 {
   if (texfilename == "")
   {
