@@ -33,6 +33,12 @@
 extern puFont FONT_HELVETICA_14;
 extern puFont FONT_SANS_12B;
 
-
+namespace Video
+{
+//Declaration of fonts defined in grapĥics.cpp.
+extern fntTexFont    *textureFont; //textured font  
+extern std::string        codeSet; //for font filename construct
+ 
+}
 #endif  // FONTS_H_
 
