@@ -34,7 +34,6 @@
 #include <math.h>
 #include <stdio.h>
 #include <plib/ssg.h>   // for ssgSimpleState
-#include "../mod_windfield_config.h"
 #include "../mod_misc/ls_constants.h"
 //jwtodo #include "../config.h"
 // remaining calls to cfg:
@@ -50,7 +49,6 @@
 #include "../crrc_main.h"
 #include "../global_video.h"
 #include "thermal03/tschalen.h"
-#include "../mod_misc/crrc_rand.h"
 #include "../mod_landscape/crrc_scenery.h"
 
 #if (THERMAL_CODE == 1)
@@ -81,11 +79,6 @@
  */
 unsigned int ThermalVersion;
 
-/**
- * Variation of wind velocity
- */
-double dWindVelVar;
-
 /**
  * There is some 2D grid. Its area is
  *    (occupancy_grid_size * occupancy_grid_res)^2
@@ -117,6 +110,14 @@ int num_thermals;
  */
 Thermal* thermals = NULL;
 
+/**
+ * Random normal (gaussian) distribution of thermal radius, strength
+ * and lifetime.
+ */
+RandGauss rnd_radius;
+RandGauss rnd_strength;
+RandGauss rnd_lifetime;
+
 /**
  * One thermal influences an area of
  *    (nInfluenceDist*occupancy_grid_res)^2
@@ -149,25 +150,6 @@ const float flThermalDistMax = 1000;
  */
 ThermikSchalen thermalv3;
 
-/**
- * There is a three-dimensional turbulence grid.
- * Little code is present yet. Variables are initialized,
- * but never do anything to the simulation, so I exclude it
- * using #if.
- * Does anyone remember who did this? Can it be finished?
- * Jens Wilhelm Wulf, 17.01.2005
- */
-#define USE_TURB_GRID 0
-#if (USE_TURB_GRID != 0)
-# define turb_grid_size 1024
-float turb_x_velocity[turb_grid_size];
-float turb_y_velocity[turb_grid_size];
-float turb_z_velocity[turb_grid_size];
-float turb_x_grad[turb_grid_size];
-float turb_y_grad[turb_grid_size];
-float turb_z_grad[turb_grid_size];
-#endif
-
 #if (THERMAL_CODE == 1)
 /**
  * Thermals get weaker before they die. Time in seconds.
@@ -226,42 +208,6 @@ inline float gridToAbsCoor(int gridKoor, float rel=0.5)
          );
 }
 
-/**
- * Returns random numbers with normal (gaussian) distribution.
- *
- * Thanks to rhoads@paul.rutgers.edu for this code!
- * http://remus.rutgers.edu/~rhoads/Code/code.html
- */
-double gaussrand()
-{
-  static double V2, fac;
-  static int phase = 0;
-  double S, Z, U1, U2, V1;
-
-  if (phase)
-    Z = V2 * fac;
-  else
-  {
-    do
-    {
-      U1 = (double)rand() / RAND_MAX;
-      U2 = (double)rand() / RAND_MAX;
-
-      V1 = 2 * U1 - 1;
-      V2 = 2 * U2 - 1;
-      S = V1 * V1 + V2 * V2;
-    }
-    while(S >= 1);
-
-    fac = sqrt (-2 * log(S) / S);
-    Z = V1 * fac;
-  }
-
-  phase = 1 - phase;
-
-  return Z;
-}
-
 /**
  * Returns true if there is a thermal nearby.
  * <code>xcoord</code> and <code>ycoord</code> are the indices into
@@ -393,8 +339,9 @@ void initialize_wind_field(SimpleXMLTransfer* el)
   Thermal *temp_thermal;
   int xloop,yloop;
 
-  dWindVelVar = 1;
-
+  // initialize wind turbulence model
+  initialize_gust();
+  
   ThermalVersion = THERMAL_CODE;
   // Use version 3?
   {
@@ -478,6 +425,11 @@ void initialize_wind_field(SimpleXMLTransfer* el)
     }
   }
 
+  // Initialise thermal random parameters distribution
+  rnd_radius.SetSigmaAndMean( cfg->thermal->radius_sigma, cfg->thermal->radius_mean );
+  rnd_strength.SetSigmaAndMean( cfg->thermal->strength_sigma, cfg->thermal->strength_mean );
+  rnd_lifetime.SetSigmaAndMean( cfg->thermal->lifetime_sigma, cfg->thermal->lifetime_mean );
+
   // Create the said number of thermals as a linked list.
   thermals = NULL;
   for (loop=0;loop<num_thermals;loop++)
@@ -488,24 +440,8 @@ void initialize_wind_field(SimpleXMLTransfer* el)
     temp_thermal->next_thermal = thermals;
     thermals = temp_thermal;
   }
-
-#if (USE_TURB_GRID != 0)
-  // Fill turbulence grid
-  turb_x_velocity[0]=0;
-  turb_y_velocity[0]=0;
-  turb_z_velocity[0]=0;
-  for (loop=1;loop<turb_grid_size;loop++)
-  {
-    turb_x_velocity[loop]=turb_x_velocity[loop-1]+(0.03*gaussrand());
-    turb_y_velocity[loop]=turb_y_velocity[loop-1]+(0.03*gaussrand());
-    turb_z_velocity[loop]=turb_y_velocity[loop-1]+(0.03*gaussrand());
-    turb_x_grad[loop]=gaussrand();
-    turb_y_grad[loop]=gaussrand();
-    turb_z_grad[loop]=gaussrand();
-  }
-#endif
-
-  therm_quadric = gluNewQuadric();  
+  
+  therm_quadric = gluNewQuadric();
 }
 
 SimpleXMLTransfer* GetDefaultConf_Thermal()
@@ -549,7 +485,7 @@ void update_thermals(float flDeltaT)
   float x_motion;   // How much has a thermal moved in X in the last timestep
   float y_motion;   // How much has a thermal moved in Y in the last timestep
   float x_wind_velocity,y_wind_velocity;
-  float flWindVel = dWindVelVar * cfg->wind->getVelocity();
+  float flWindVel = cfg->wind->getVelocity();
 
   // Wind velocity is the same everywhere, so every thermals relative movement is:
   x_wind_velocity = -1 * flWindVel * cos(M_PI*cfg->wind->getDirection()/180);
@@ -570,12 +506,13 @@ void update_thermals(float flDeltaT)
 int calculate_wind(double  X_cg,      double  Y_cg,     double  Z_cg,
                    double& Vel_north, double& Vel_east, double& Vel_down)
 {
-  float    x_wind_velocity,y_wind_velocity,z_wind_velocity;//JL
+  float    x_wind_velocity, y_wind_velocity, z_wind_velocity; //JL
   Thermal* thermal_ptr;
   int      aircraft_xcoord,aircraft_ycoord;
   int      xloop,yloop;
-  double   thermal_wind_x=0;
-  double   thermal_wind_y=0;
+  double   thermal_wind_x = 0;
+  double   thermal_wind_y = 0;
+  double   thermal_wind_z = 0;
 #if (THERMAL_CODE == 0)
   double distance_from_core;
   float total_up_airmass = 0;
@@ -585,16 +522,33 @@ int calculate_wind(double  X_cg,      double  Y_cg,     double  Z_cg,
   float lift_area  = 0;
   int   in_thermal = FALSE;
   float angle_in;
-  float v_in_max;      // Max velocity of thermal vacuum cleaner wind.
+  float v_in_max; // Max velocity of thermal vacuum cleaner wind.
 #endif
 
-  Vel_north = 0;
-  Vel_east  = 0;
-  Vel_down  = 0;
-  Global::scenery->getWindComponents(X_cg, Y_cg, Z_cg, &x_wind_velocity, &y_wind_velocity, &z_wind_velocity);
-  x_wind_velocity *= dWindVelVar;
-  y_wind_velocity *= dWindVelVar;
-  z_wind_velocity *= dWindVelVar;
+  // wind from scenery, without thermal effect
+  int wind_error = Global::scenery->getWindComponents(X_cg, Y_cg, Z_cg, 
+      &x_wind_velocity, &y_wind_velocity, &z_wind_velocity);
+
+  // reduce wind speed close to ground to simulate wind 
+  // boundary layer effect, using a 1/7th power law + linear sublayer
+  float layer_thickness = 30.0; // boundary layer thickness set to 30ft (10m)
+  float sublayer_thickness = 0.05; // relative thickness of linear sublayer (1.5ft, 0.5m)
+  float Z_ter = Global::scenery->getHeight(X_cg, Y_cg); //terrain height below the point
+  float Z_scale = (-Z_cg - Z_ter)/layer_thickness; //remember: Z_cg is positive down
+  float fact;
+  if (Z_scale < 0.0)
+    Z_scale = 0.0;
+  if (Z_scale > 1.0)
+    Z_scale = 1.0;
+  if (Z_scale < sublayer_thickness)
+    fact = Z_scale*pow(sublayer_thickness,-6./7.);
+  else
+    fact = pow(Z_scale,1./7.);
+  
+  Vel_north = fact*x_wind_velocity;
+  Vel_east  = fact*y_wind_velocity;
+  Vel_down  = fact*z_wind_velocity;
+ 
   // calculate indices of the aircraft in the grid
   aircraft_xcoord = absToGridCoor(X_cg);
   aircraft_ycoord = absToGridCoor(Y_cg);
@@ -623,7 +577,7 @@ int calculate_wind(double  X_cg,      double  Y_cg,     double  Z_cg,
            case 3:
             thermal_ptr->sumVelocity(X_cg, Y_cg, Z_cg,
                                      thermalv3,
-                                     thermal_wind_x, thermal_wind_y, Vel_down);
+                                     thermal_wind_x, thermal_wind_y, thermal_wind_z);
             break;
 
            default:
@@ -635,7 +589,7 @@ int calculate_wind(double  X_cg,      double  Y_cg,     double  Z_cg,
             total_up_airmass+=thermal_area*thermal_ptr->strength;
 #endif
 #if (THERMAL_CODE == 1)
-            Vel_down += thermal_ptr->getVelocity(X_cg, Y_cg, Z_cg);
+            thermal_wind_z += thermal_ptr->getVelocity(X_cg, Y_cg, Z_cg);
 #endif
             break;
           }
@@ -698,12 +652,12 @@ int calculate_wind(double  X_cg,      double  Y_cg,     double  Z_cg,
 
             if (distance_from_core < thermal_ptr->radius)
             {
-              Vel_down  = -1*thermal_ptr->strength;
+              thermal_wind_z = -1*thermal_ptr->strength;
               in_thermal = TRUE;
             }
             else if (distance_from_core < thermal_ptr->radius+thermal_ptr->boundary_thickness)
             {
-              Vel_down  = -1*thermal_ptr->strength
+              thermal_wind_z = -1*thermal_ptr->strength
                 + (thermal_ptr->strength+sink_strength)*((distance_from_core-thermal_ptr->radius)/thermal_ptr->boundary_thickness);
               in_thermal = TRUE;
             }
@@ -713,32 +667,178 @@ int calculate_wind(double  X_cg,      double  Y_cg,     double  Z_cg,
       // end of loop
       //
       // If this is not in a thermal, sink_strength is used. If this is in a
-      // thermal, Vel_down has been set in the loop above.
+      // thermal, thermal_wind_z has been set in the loop above.
       if (!in_thermal)
       {
-        Vel_down= sink_strength;
+        thermal_wind_z = sink_strength;
       }
 
       // thermals grow stronger from the ground up
       if (-Z_cg < 50)
       {
-        Vel_down *= (-Z_cg/50);
+        thermal_wind_z *= (-Z_cg/50);
       }
 #endif
       break;
     }
 
-    Vel_down += z_wind_velocity;
-	  Vel_north= x_wind_velocity + thermal_wind_x;
-    Vel_east = y_wind_velocity + thermal_wind_y;
+	  Vel_north += thermal_wind_x;
+    Vel_east  += thermal_wind_y;
+    Vel_down  += thermal_wind_z;
+  }
 
-    return(0);
-  }
-  else
+  return wind_error;
+}
+
+// Description: see header file
+int calculate_wind_grad(double X_cg, double Y_cg, double Z_cg, double delta_space,
+                        CRRCMath::Matrix33& m_V_grad)
+{
+  double V_north_xp, V_east_xp, V_down_xp;
+  double V_north_yp, V_east_yp, V_down_yp;
+  double V_north_zp, V_east_zp, V_down_zp;
+  double V_north_xm, V_east_xm, V_down_xm;
+  double V_north_ym, V_east_ym, V_down_ym;
+  double V_north_zm, V_east_zm, V_down_zm;
+
+  int err_x = calculate_wind(X_cg+delta_space, Y_cg,             Z_cg,
+                             V_north_xp,       V_east_xp,        V_down_xp) |
+              calculate_wind(X_cg-delta_space, Y_cg,             Z_cg,
+                             V_north_xm,       V_east_xm,        V_down_xm);
+  int err_y = calculate_wind(X_cg,             Y_cg+delta_space, Z_cg,
+                             V_north_yp,       V_east_yp,        V_down_yp) |
+              calculate_wind(X_cg,             Y_cg-delta_space, Z_cg,
+                             V_north_ym,       V_east_ym,        V_down_ym);
+  int err_z = calculate_wind(X_cg,             Y_cg,             Z_cg+delta_space,
+                             V_north_zp,       V_east_zp,        V_down_zp) |
+              calculate_wind(X_cg,             Y_cg,             Z_cg-delta_space,
+                             V_north_zm,       V_east_zm,        V_down_zm);
+
+  // Gradients are calculated from symmetric pairs to get symmetric behaviour.
+  if (!err_x)
   {
-    // The aircraft is out of the grid.
-    return(1);
+    m_V_grad.v[0][0] = (V_north_xp - V_north_xm)/(2*delta_space);
+    m_V_grad.v[1][0] = (V_east_xp  - V_east_xm) /(2*delta_space);
+    m_V_grad.v[2][0] = (V_down_xp  - V_down_xm) /(2*delta_space);
   }
+  if (!err_y)
+  {
+    m_V_grad.v[0][1] = (V_north_yp - V_north_ym)/(2*delta_space);
+    m_V_grad.v[1][1] = (V_east_yp  - V_east_ym) /(2*delta_space);
+    m_V_grad.v[2][1] = (V_down_yp  - V_down_ym) /(2*delta_space);
+  }
+  if (!err_z)
+  {
+    m_V_grad.v[0][2] = (V_north_zp - V_north_zm)/(2*delta_space);
+    m_V_grad.v[1][2] = (V_east_zp  - V_east_zm) /(2*delta_space);
+    m_V_grad.v[2][2] = (V_down_zp  - V_down_zm) /(2*delta_space);
+  }
+
+  return (err_x | err_y | err_z);
+}
+
+// Description: see header file
+void initialize_gust()
+{
+  v_V_gust_body_.r[0] = v_V_gust_body_.r[1] = v_V_gust_body_.r[2] = 0.0;
+  v_V_gust_body_old_ = v_V_gust_body_;
+
+  v_R_omega_gust_body_.r[0] = v_R_omega_gust_body_.r[1] = v_R_omega_gust_body_.r[2] = 0.0;
+}
+
+// Description: see header file
+void calculate_gust(double dt, double altitude, double V_rel_wind, double b,
+                    CRRCMath::Vector3 v_V_local_airmass,
+                    CRRCMath::Matrix33 LocalToBody,
+                    CRRCMath::Vector3& v_V_gust_body,
+                    CRRCMath::Vector3& v_R_omega_gust_body)
+{
+  double intensity = cfg->wind->getTurbulence();
+  double V_wind = v_V_local_airmass.length();
+
+  // no wind turbulence if wind velocity is zero or
+  // relative turbulence intensity has been set to zero
+  if ( intensity * V_wind == 0.)
+    return;
+  
+  v_V_local_airmass.normalize();
+  CRRCMath::Matrix33 WindToLocal(v_V_local_airmass.r[0], -v_V_local_airmass.r[1], 0.,
+                                 v_V_local_airmass.r[1],  v_V_local_airmass.r[0], 0.,
+                                 0.,                      0.,                     1.);
+  
+  // linear and rotational gust velocity estimated using digital filter 
+  // form of Dryden spectra, from MIL-HDBK-1797.
+  // NB: sigma and reference length from low-altitude specification
+
+  v_V_gust_body_old_ = v_V_gust_body_;
+
+  double V_dt = dt*V_rel_wind;
+  double alt = altitude < 1000. ? (altitude > 10. ? altitude : 10) : 1000.;
+  double factor = 1.0/(0.177 + 0.000823*alt);
+
+  double Lu = alt*pow(factor,1.2);
+  double Lv = 0.5*Lu;
+  double Lw = 0.5*alt;
+  
+  double pid4b = M_PI/(4.0*b);
+  double pid3b = M_PI/(3.0*b);
+  
+  double sigw = 0.1*V_wind;
+  double sigu = sigw*pow(factor,0.4);
+  double sigv = sigu;
+  
+  // align length scale "u" with wind direction, then
+  // transform length scale from local to body frame
+  CRRCMath::Matrix33 L_tensor(Lu, 0., 0.,
+                              0., Lv, 0.,
+                              0., 0., Lw);
+  L_tensor = WindToLocal*(L_tensor*WindToLocal.trans());
+  L_tensor = LocalToBody*(L_tensor*LocalToBody.trans());
+  Lu = L_tensor.v[0][0];
+  Lv = L_tensor.v[1][1];
+  Lw = L_tensor.v[2][2];
+
+  // align sigma "u" with wind direction, then
+  // transform sigma from local to body frame
+  CRRCMath::Matrix33 s_tensor(sigu, 0., 0.,
+                              0., sigv, 0.,
+                              0., 0., sigw);
+  s_tensor = WindToLocal*(s_tensor*WindToLocal.trans());
+  s_tensor = LocalToBody*(s_tensor*LocalToBody.trans());
+  sigu = s_tensor.v[0][0];
+  sigv = s_tensor.v[1][1];
+  sigw = s_tensor.v[2][2];
+
+  double temp = sqrt(2.0*Lw*b);
+  double Lp   = temp/2.6;
+  double sigp = 1.9*sigw/temp;
+
+  double au_dt = V_dt/Lu;
+  double av_dt = V_dt/Lv;
+  double aw_dt = V_dt/Lw;
+  double ap_dt = V_dt/Lp;
+  double aq_dt = pid4b*V_dt;
+  double ar_dt = pid3b*V_dt;
+  
+  v_V_gust_body_.r[0]       = (1.0 - au_dt)*v_V_gust_body_.r[0] 
+                              + sqrt(2.0*au_dt)*sigu*eta1.Get();
+  v_V_gust_body_.r[1]       = (1.0 - av_dt)*v_V_gust_body_.r[1]
+                              + sqrt(2.0*av_dt)*sigv*eta2.Get();
+  v_V_gust_body_.r[2]       = (1.0 - aw_dt)*v_V_gust_body_.r[2]
+                              + sqrt(2.0*aw_dt)*sigw*eta3.Get();                             
+
+  // NB: signs for q and r (airmass turbulence rotation around body y and z)
+  //     are consistent with airmass rotation computed from airmass velocity
+  //     gradient (see e.g. fdm_larcsim.cpp). Sign for p is arbitrary.
+  v_R_omega_gust_body_.r[0] = (1.0 - ap_dt)*v_R_omega_gust_body_.r[0] 
+                              + sqrt(2.0*ap_dt)*sigp*eta4.Get();
+  v_R_omega_gust_body_.r[1] = (1.0 - aq_dt)*v_R_omega_gust_body_.r[1]
+                              - pid4b*(v_V_gust_body_.r[2] - v_V_gust_body_old_.r[2]);
+  v_R_omega_gust_body_.r[2] = (1.0 - ar_dt)*v_R_omega_gust_body_.r[2]
+                              + pid3b*(v_V_gust_body_.r[1] - v_V_gust_body_old_.r[1]);
+                             
+  v_V_gust_body = v_V_gust_body_ * intensity;
+  v_R_omega_gust_body = v_R_omega_gust_body_ * intensity;
 }
 
 // Description: see header file
@@ -799,7 +899,7 @@ void draw_thermals(CRRCMath::Vector3 pos)
 
 void draw_wind(double direction_face)
 {
-  double length    = 0.8*dWindVelVar;
+  double length    = 0.8;
   double direction = (M_PI*cfg->wind->getDirection()/180) - direction_face;
 
   //
@@ -1041,25 +1141,22 @@ void Thermal::random_init()
   // Describe thermal
   center_x_position = xpos;
   center_y_position = ypos;
-  xcoord            = xco;
-  ycoord            = yco;
-  radius=(gaussrand()*cfg->thermal->radius_sigma)+cfg->thermal->radius_mean;
+  xcoord = xco;
+  ycoord = yco;
+  radius   = rnd_radius.Get();
+  strength = rnd_strength.Get();
+  lifetime = rnd_lifetime.Get();
+#if THERMAL_TEST != 0
+  radius   = 50;
+  strength = 15;
+  lifetime = 9999;
+#endif
 #if (THERMAL_CODE == 0)
   // todo: is this boundary thickness correct? Until 2005-01-15 the initial thermals
   // have not been created using this code. It was radius/5 there.
   // 2005-01-20: gradient is very high -- using /5 now.
   boundary_thickness = radius/5;
 #endif
-  strength=(gaussrand()*cfg->thermal->strength_sigma)+
-    cfg->thermal->strength_mean;
-  lifetime=(gaussrand()*cfg->thermal->lifetime_sigma)+
-    cfg->thermal->lifetime_mean;
-
-#if THERMAL_TEST != 0
-  radius   = 50;
-  strength = 15;
-  lifetime = 9999;
-#endif
 
   switch (ThermalVersion)
   {
